#!/bin/bash
#
# Script to interrogate a stick and mount it appropriately depending on what is on it.
# 
# This value (sdb1, sdb2...) is passed in by /etc/init.d/10-media-usb.rules
# 
#
#

VA_HINT_FILE="va"
MEDIA_XML="media.xml"

# Mount a VA content stick if we see the va hint file on it.
mount_va()
{
    VA_USB_MOUNTPOINT="/netpulse/data/va-media"

    # Check if the mount point exists
    if [ -d $VA_USB_MOUNTPOINT ]; then 
        echo "$VA_USB_MOUNTPOINT exists"
    else 
        echo "Directory does not exists, create our usb mount point."
        mkdir -p $VA_USB_MOUNTPOINT    
        #chown netpulse:netpulse $VA_USB_MOUNTPOINT
    fi 

    echo "Mounting VA content: $USB_DEV $VA_USB_MOUNTPOINT"
    /bin/mount -t vfat,ext3,ext4 $USB_DEV $VA_USB_MOUNTPOINT
    #chown -R netpulse:netpulse $VA_USB_MOUNTPOINT
    
}

check_content_mounted() 
{
    mount_point="$1"
    mount | grep -v grep | grep $mount_point >/dev/null 2>&1
}

mount_content()
{
    if [ -f /technogym/netpulse/data/machine.xml ]; then 
        REAL_USB_MOUNTPOINT=/technogym/netpulse/data/prod
        OWNER=root
    elif [ -f /npboot/netpulse/data/machine.xml ]; then
        REAL_USB_MOUNTPOINT=/npboot/netpulse/data/prod
        OWNER=root
    elif [ -f /netpulse/www/data/machine.xml ]; then
        REAL_USB_MOUNTPOINT=/clips
        OWNER=netpulse
    elif [ -f /netpulse/data/machine.xml ]; then
        REAL_USB_MOUNTPOINT=/netpulse/data/prod
        OWNER=`who am i | cut -f 1 -d ' '`
    else
        echo "No value netpulse installation found"
        exit
    fi

    if [ -d $REAL_USB_MOUNTPOINT ]; then 
        echo "$REAL_USB_MOUNTPOINT exists"
    else 
        echo "Directory does not exists, create our usb mount point."
        mkdir -p $REAL_USB_MOUNTPOINT    
        chown $OWNER:$OWNER $REAL_USB_MOUNTPOINT
    fi 

    MUID=`id -u $OWNER`
    MGID=`id -g $OWNER`

    echo "$MUID $MGID $USB_DEV $REAL_USB_MOUNTPOINT"

    /bin/mount -r -o uid=$MUID,gid=$MGID -t vfat $USB_DEV $REAL_USB_MOUNTPOINT
    if [ $? -ne 0 ]
    then
        echo "Could not mount $USB_DEV to $REAL_USB_MOUNTPOINT.  Trying read/write instead of read-only.  Please be sure this machine is not on the network."
        /bin/mount -o uid=$MUID,gid=$MGID -t vfat $USB_DEV $REAL_USB_MOUNTPOINT
    fi
    check_content_mounted $REAL_USB_MOUNTPOINT && echo "Content mounted OK."
}

# We are going to mount the device to a temporary directory
# then depending on what we find (va or content) remount it 
# to the right spot

pre_mount()
{
    TMP_USB_MOUNTPOINT=/tmp/checkstick

    if [ -d $TMP_USB_MOUNTPOINT ]; then
        echo "$TMP_USB_MOUNTPOINT exists"
    else 
        echo "Directory does not exists, create our usb mount point."
        mkdir -p $TMP_USB_MOUNTPOINT	
    fi 

    mount $USB_DEV $TMP_USB_MOUNTPOINT
    echo "Mounted $USB_DEV to tmp point: $TMP_USB_MOUNTPOINT"

    if [ -e ${TMP_USB_MOUNTPOINT}/$VA_HINT_FILE ]; then
        echo "OK: Valid Virtual Active USB!: "
        /bin/umount -f $USB_DEV
        mount_va
    elif [ -e ${TMP_USB_MOUNTPOINT}/${MEDIA_XML} ]; then
        echo "OK: Valid content. found media.xml: "
        /bin/umount -f $USB_DEV
        mount_content 
    else 
        echo "No content that we understand here"
        /bin/umount -f $USB_DEV
    fi
}

if [ $# -ne 1 ]; then
    echo "Mount point or valid argument not specified. (exit)"
    exit
fi

if [ "$1" == "-f" ]; then
    echo "Detecting USB storage ..."

    for USB_DISK_BY_ID in `ls -1 /dev/disk/by-id/usb-* 2>/dev/null | grep "part[1-9]"`
    do
        echo "Found device: $USB_DISK_BY_ID Getting block device ..."
        USB_DEV=`readlink -e $USB_DISK_BY_ID`
        if [ -n "$USB_DEV" ] ; then
            echo "block device: interrogating: $USB_DEV"
            pre_mount 
        fi
    done
else
    USB_DEV="/dev/$1"
    pre_mount
fi
