
//
//  Defensive AJAX wrapper.
//
function dirtyHarry(reqURL, successCallback, errorCallback, continueCallback, reqTimeout, reqType, reqDataType, reqDomain, attempts) {
    try {
        //  If URL is not specified - don't do anything and report an error.
        if(!reqURL)
            throw("No URL");

		// Make sure an ajax request for reqURL is not pending before trying to send it.
		if (!window.pendingRequests) {
			pendingRequests = {};
		} else {
			if (pendingRequests[reqURL]) {
				hidePasscode(reqURL, 'pending');
				return;
			}
		}

        //  Initialize variables - set to defaults if not specified.
        var successCallback = (successCallback) ? successCallback : null;       //  Success callback - default to null.
        var errorCallback = (errorCallback) ? errorCallback : null;             //  Error callback - default to null.
        var continueCallback = (continueCallback) ? continueCallback : null;    //  Continue callback - default to null.
        var reqTimeout = (reqTimeout) ? reqTimeout : 5*1000;                    //  Use 5 second timeout by default
        var reqType = (reqType) ? reqType : "GET";                              //  Use GET request type by default
        var reqDataType = (reqDataType) ? reqDataType.toLowerCase() : "xml";    //  Use XML data type by default
        var reqDomain = (reqDomain) ? reqDomain : false;							//	Use crossDomain access. 
        var attempts = (!attempts) ? 1 : attempts;
        var attemptsCounter = 1;
        
        //  Log every AJAX request.

        //hidePasscode(reqURL,'calling');

        ajaxCall();
        function ajaxCall(){
            //  AJAX request
            $.ajax({
            	crossDomain:reqDomain,
                type:		reqType,
                url:		reqURL,
                dataType:	reqDataType,
                timeout:	reqTimeout,
                cache:		true,

                //
                //  If the AJAX request comes back successfully, call successCallback() if specified.
                //  If there are any errors while running successCallback(), or if the server response
                //  code is not 0 (success) (TODO), call errorCallback() if specified.
                //
                success: function dirtyHarrySuccess(data, textStatus, XMLHttpRequest) {
                    hidePasscode(reqURL, 'request'); 
                    try {
    		            if (successCallback) {
    		                successCallback(data, textStatus, XMLHttpRequest);
    					}
                    } catch(e) { popError(e); }
                },

                //
                //  If the AJAX request fails, call popError() with a string detailing the problem.
                //
                error: function dirtyHarryError(XMLHttpRequest, textStatus, errorThrown) {
                    try {
						if(errorCallback){
							if(textStatus == "timeout" && attemptsCounter < attempts){
								healthObj.reset_activity();
								attemptsCounter++;
	                			ajaxCall();
		                	} else{
								hidePasscode(reqURL,'error',textStatus);
		                		healthObj.reset_activity();
								errorCallback(XMLHttpRequest, textStatus, errorThrown);
		                	}
						}
    		            
                    } catch(err) { popError(err); }
                },

                //
                //  This function will be called regardless of request success/failure, and is called after the success
                //  and error functions (like a "finally" block).
                //
                complete: function dirtyHarryContinue(XMLHttpRequest, textStatus) {
                    try {
    				
    				if (window.pendingRequests) 
    					delete pendingRequests[reqURL];
    					
                    if(continueCallback)
                        continueCallback();

                    } catch(err) { popError(err); }
                }
            });
        }
        
        pendingRequests[reqURL] = true;
    }

    //
    //  If the main block fails, just do a normal popError().
    //
    catch(e) {
        var errorStr;
        switch(e) {
            case "No URL":
                errorStr = "No request URL specified";
                break;
            default:
                errorStr = e;
                break;
        }
        popError(errorStr);
    }
}

function hidePasscode(reqURL, whichRequest, textStatus){

    var checkPasscode = reqURL.search("&passcode");
    var checkNewPasscode = reqURL.search("&newPasscode");
    var reqArray = reqURL.split("&"); 
    var suppressReq = [];
    var theString = suppressReq.toString();

    if(whichRequest == 'error'){

        if(checkPasscode != -1 || checkNewPasscode != -1){
            for(i=0; i<4; i++){
                suppressReq.push(reqArray[i]);
            }
            var newSuppressString = suppressReq.toString();  
            var finalSuppressString = newSuppressString.replace(',','&').replace(',pic=','&pic=');
            popError("AJAX request -"+finalSuppressString+" - Status: "+textStatus); 

        }else{

            popError("AJAX request -"+reqURL+" - Status: "+textStatus); 
        }
    }

    if(checkPasscode != -1 || checkNewPasscode != -1){
        for(i=0; i<4; i++){
            suppressReq.push(reqArray[i]);
        }
        var newSuppressString = suppressReq.toString();  
        var finalSuppressString = newSuppressString.replace(',','&').replace(',pic=','&pic=');
            
        if(textStatus != undefined){
          writeLog("info", "AJAX request for " +finalSuppressString+ "&passcode=**** - Error: "+textStatus);   
        }else{
            writeLog("info", "AJAX request for " +finalSuppressString+ "&passcode=**** successful!");
        }
    }else{

        if(textStatus != undefined){
          writeLog("info", "AJAX request for " + reqURL + " - Error: "+textStatus);   
        }else{
            writeLog("info", "AJAX request for " + reqURL + " successful!"); 
        }
    }
}






