
//
//  Test masterTimer.js as web worker
//

var masterTimerInterval = null;
var tickSpeedMS = 100;
var counters = new Array();

//
//  Add new counter to counters[]
//
function addCounter(counter)
{
    counters.push(counter);
    figureAllTickSpeeds();
}

//
//  Remove the specified counter from counters[]
//
function removeCounter(counterName)
{
    for(var i=0;i<counters.length;i++)
    {
        if(counters[i].name == counterName)
        {
            postMessage(["removed", counters[i].name]);
            counters.splice(i, 1);
            break;
        }
    }

    figureAllTickSpeeds();
}

//
//  Remove all intervals with a given namespace (defined via "." in the interval name)
//
function clearNamespace(namespace)
{
    for(var i=0,thisNameArr="";i<counters.length;i++)
    {
        thisNameArr = counters[i].name.split(".");

        //  If this interval doesn't have a namespace, skip it.
        if(thisNameArr.length <= 1)
            continue;

        //  If namespace matches, remove this interval and tell the main master timer control to remove it as well.
        if(thisNameArr[1] == namespace)
        {
            postMessage(["removed", counters[i].name]);
            counters.splice(i, 1);
            clearNamespace(namespace);
            return;
        }
    }
}

//
//  Update master timer speed
//
function updateRate(rate)
{
    tickSpeedMS = rate;
}

//
//  Increment the count value for each counter in counters[]. If the count value reaches a counter's tick value, post a
//  message back to the main app triggering that function.
//
function masterTimerTick()
{
    for(var i=0;i<counters.length;i++)
    {
        counters[i].count++;
        if(counters[i].count < counters[i].tick)
            continue;

        postMessage(["tick", counters[i]]);
        counters[i].count = 0;
    }
}

function figureAllTickSpeeds()
{
    for(var i=0;i<counters.length;i++)
        counters[i].tick = figureTicks(counters[i]);
}

function figureTicks(counter)
{
    return(counter.rate/tickSpeedMS);
}

function init() { counters = new Array(); }

function start()
{
    resetMasterTimer();
    masterTimerInterval = setInterval( function() { masterTimerTick(); }, tickSpeedMS );
}

function end()
{
    resetMasterTimer();
    counters = new Array();
    postMessage("dead");
}

function resetMasterTimer()
{
    clearInterval(masterTimerInterval);
    masterTimerInterval = null;
}

function errorReceiver(event) { throw event.data; }

//
//  Message is received from the main app.
//
onmessage = function(event)
{
    var m = (typeof(event.data) == "string") ? event.data : event.data[0];

    switch(m)
    {
        case "init":
            init();
            break;

        case "add":
            addCounter(event.data[1]);
            break;

        case "remove":
            removeCounter(event.data[1]);
            break;

        case "clearNamespace":
            clearNamespace(event.data[1]);
            break;

        case "rate":
            updateRate(event.data[1]);
            break;

        case "begin":
            start();
            break;

        case "pause":
            resetMasterTimer();
            break;

        case "die":
            end();
            break;

        default:
            break;
    }
};


