// TV Configuration presented as a global/static object, so that parsing 
// channels is processed once and kept in memory 
var tvConfig = {
	allChannels     : [],
	enabledChannels : []
};

tvConfig.getEmbeddedChannelList = function() {
	if (tvConfig.allChannels.length == 0) {
		// Get customized channel data from tv_conf.xml
		var fileStr = read_file(DIR_ROOT + "/data/tv_conf.xml");
		var $channels = $((new DOMParser()).parseFromString(fileStr, 'text/xml').getElementsByTagName("channels")[0]);
		
		// Go thru each customized channel, adding it to allChannels with the customized attributes
		var i = 0;
		$channels.find("channel").each(function tvConfigSetEachChannel_embedded() {
			if ($(this).attr("number")) {
				if ($(this).attr("enabled") && $(this).attr("enabled").toLowerCase() == "true") {
					tvConfig.allChannels.push({
						channel          : $(this).attr("number"),
						name             : $(this).attr("name"),
						position         : $(this).attr("position"),
						icon             : $(this).attr("icon"),
						id				 : $(this).attr("id"),
						extra			 : $(this).attr("extra"),
						originalPosition : String(i), // Original c index file that never changes. Not revelant for matrix 7xi
						enabled          : true
					});	
					i++;
				}
			}
		});
		
		tvConfig.allChannels.sort(tvConfig.sortByChannelPosition);
		
		// All channels in the allChannels array will be enabled.
		tvConfig.enabledChannels = tvConfig.allChannels;
	}

	return tvConfig.allChannels;
}

tvConfig.getStandaloneChannelList = function(skipDisabledChannels) {
	if (tvConfig.allChannels.length == 0) {
		
		// Begin with channel list from C code
		var tvJSON = evalJSON(getChannelListJSON());
		
		for (var i=0; i<tvJSON.total; i++) {
			var thisChannel = tvJSON.channel[i];
			
			// Analog channels have extra info encoded in their channel name.
			// Format is name:freq|channelNum:position
			channelInfo = String(thisChannel).split(":");
			
			tvConfig.allChannels.push({
				channel          : (getChannelType(String(i)) == "0") ? channelInfo[2] : channelInfo[0],
				name             : channelInfo[0],
				position         : i,
				icon             : "",
				originalPosition : String(i), // Original c index file that never changes
				enabled          : true
			});
		}
		
		// Get customized channel data from tv_conf.xml
		var fileStr = read_file(DIR_ROOT + "/data/tv_conf.xml");
		var $channels = $((new DOMParser()).parseFromString(fileStr, 'text/xml').getElementsByTagName("channels")[0]);
		
		// Go thru each customized channel, find the matching channel in allChannels[], and add customized attributes
		$channels.find("channel").each(function tvConfigSetEachChannel_standalone() {
			for (var i=0; i<tvConfig.allChannels.length; i++) {
				if ($(this).attr("number")) {
					if ($(this).attr("number").toLowerCase() != tvConfig.allChannels[i].channel.toLowerCase())
						continue;
					
					if ($(this).attr("enabled") && $(this).attr("enabled").toLowerCase() == "false")
						tvConfig.allChannels[i].enabled = false;
					
					tvConfig.allChannels[i].name = ($(this).attr("name")) ? $(this).attr("name") : tvConfig.allChannels[i].name; // TODO: default to channel?
					tvConfig.allChannels[i].position = ($(this).attr("position")) ? $(this).attr("position") : i;
					tvConfig.allChannels[i].icon = $(this).attr("icon");
				}
			}
		});
		
		tvConfig.allChannels.sort(tvConfig.sortByChannelPosition);
	}

	if (skipDisabledChannels) {
		if (tvConfig.enabledChannels.length == 0) {
			for (var i=0; i<tvConfig.allChannels.length; i++) {
				if (tvConfig.allChannels[i].enabled) 
					tvConfig.enabledChannels.push(tvConfig.allChannels[i]);
			}
			tvConfig.enabledChannels.sort(tvConfig.sortByChannelPosition);
		}
		return tvConfig.enabledChannels;
	} else
		return tvConfig.allChannels;
}

tvConfig.getChannelList = function tvConfigGetChannelList(skipDisabledChannels) {
	return (machineObj.getSlaveState() == 3) ? this.getEmbeddedChannelList() : this.getStandaloneChannelList(skipDisabledChannels);
}

function tvConfigSortByChannelPosition(a, b) {
	var ap = parseInt(a.position), bp = parseInt(b.position);
	return ( ( ap < bp ) ? -1 : ( ( ap > bp ) ? 1 : 0 ) );
}
tvConfig.sortByChannelPosition = tvConfigSortByChannelPosition;

function tvConfigClearChannelList() {
	tvConfig.allChannels = [];
	tvConfig.enabledChannels = [];
}
tvConfig.clearChannelList = tvConfigClearChannelList;