userCustomPlaylist = {
	"title" : "My Playlist",
	"shuffle" : false,
	"items" : [],
	
	"init" : function() {
		this.reset();
	},
	"reset" : function() {
		this.title = IDS_LANGUAGE["customMusicVideoPlaylist"];
		this.shuffle = false;
		this.empty();
	},
	"empty" : function() {
		var removedItems = this.__items__.splice(0, this.__items__.length);
		for (var i=0, len=removedItems.length; i<len; i++) {
			delete this.__quickLookup__[removedItems[i].id];
		}
	}, 
	"save" : function() {
		if (!this.title || !athleteObj.get_user_id()) {
			return;
		}
		
	    var urlStr = machineObj.getServerLocation() + "n5iplaylist.jsp?" +
        "mid=" + configObj.getMid() +
        "&exerciserId=" + athleteObj.get_user_id() +
        "&name=" + this.title +
        "&shuffle=" + this.shuffle +
        "&mediaIds=";

	    for(var i=0, len=this.__items__.length; i<len; i++) {
	    	if(i > 0) urlStr += ",";
	    	urlStr += this.__items__[i].id;
	    }
	    dirtyHarry(urlStr, 
    		function() { writeLog('info', 'Saved custom music playlist'); }, 
    		function() { writeLog('info', 'Error saving custom music playlist'); }
	    );
	},
	"setTitle" : function(newTitle) {
		if (newTitle && newTitle.trim()) {
			this.title = newTitle.trim();
		}
		else {
			this.title = IDS_LANGUAGE["customMusicVideoPlaylist"];
		}
	},
	"contains" : function(mediaId) {
		return (this.__quickLookup__[mediaId] >= 0) ? true : false;
			
	},
	"add" : function(mediaItem) {
		// Don't allow duplicates!
		if (this.contains(mediaItem.id)) return;
		this.__quickLookup__[mediaItem.id] = this.__items__.length;
		this.__items__.push(mediaItem);
	},
	"remove" : function(mediaId) {
		if (this.contains(mediaId)) {
			var i = this.__quickLookup__[mediaId];
			this.__items__.splice(i, 1);
			delete this.__quickLookup__[mediaId];
			
			// Removing an item will change the index of all elements after the removed one
			// resulting in the __quickLookup__ array to be out of sync
			this.__resync__();
		}
	},
	"get" : function(mediaId) {
		var i = this.__quickLookup__[mediaId];
		return (i>=0) ? this.__items__[i] : null; 
	},
	"__resync__" : function() {
		for (var i=0, len=this.__items__.length; i<len; i++) {
			this.__quickLookup__[this.__items__[i].id] = i;
		}
	}
};


// In order to control access to and modification of the custom playlist, we'll change the
// items property to be a getter/setter disguised as a variable. Notice that the getter
// returns a clone of the "true" items array, so doing something like 
// userCustomPlaylist.items[0] = "foobar" won't actually change the internal array.
// This will help keep __quickLookup__ in sync
// 
Object.defineProperty(userCustomPlaylist, "items", {
	get: function() {
		// Return a clone to prevent modifications
		return this.__items__.concat([]);  // Shallow copy
		//return JSON.parse(JSON.stringify(this.__items__);  // Deep copy
	},
	set: function(newValue) {
		var me = this;
		if (!(newValue instanceof Array)) {
			newValue = [];
		}
		var len = newValue.length;
		if (len > 0) {
			this.__items__.splice(0, this.__items__.length);
			for (var i=0; i<len; i++) {
				this.__items__.push(newValue[i]);
				this.__quickLookup__[newValue[i].id] = i;
			}
		}
	}, 
	configurable: false
});

// Make the __items__ array as private as JavaScript will allow
Object.defineProperty(userCustomPlaylist, "__items__", {
	value: [],
	writable: false,
	configurable: false,
	enumerable: false
});

// A mapping of video IDs to their index into the items array. Again, make this as
// private as JavaScript will allow
Object.defineProperty(userCustomPlaylist, "__quickLookup__", {
	value: {},
	writable: false,
	configurable: false,
	enumerable: false
});

Object.defineProperty(userCustomPlaylist, "__resync__", {
	configurable: false,
	enumerable: false,
	writable: false
});
