/*---------------------------------------------------------------------
	Client-Zone Script (core) | jsi/clientZone.js
---------------------------------------------------------------------*/
//  Client zone management code
var g_clientZoneList = [], g_clientZonesOnPage = [];

function initializeClientZones() {
	g_clientZoneList = [];
	
	// Although ajax can be used to load local files, we've experienced some issues where the call would timeout
	// before the request could be completed. Maybe the JavaScript gets overloaded? Anyway, I think's it better 
	// to use read_file. Let ajax handle over the network requests.
	//dirtyHarry(("file://"+DIR_ROOT + "/data/filelist.xml"), function clientZoneSuccess(data) { clientZoneCallback(data); });
	
	var dir = DIR_ROOT + "/data/";
    var fileName = dir + "filelist.xml";
    writeLog("info", "Looking for filelist.xml in: " + dir);

    if (file_exists(fileName) == "1") {
        var fileStr = read_file(fileName);
        var xmlObject = (new DOMParser()).parseFromString(fileStr, 'text/xml');
		clientZoneCallback(xmlObject);
    } else {
    	writeLog("info", "Unable to find filelist.xml");
    }
}

function clientZoneCallback(data) {
	$(data).find("item").each(function clientZoneCallbackItem() {
		if ($(this).attr("name").split("-").length > 1) {
			var thisImage = new Image();
			thisImage.src = DIR_ROOT + "/data/" + $(this).attr("name");
			var thisZone = thisImage.src.split("-")[0];
			g_clientZoneList.push({
				image : thisImage,
				name : thisZone
			});
		}
	});
}

function setupClientZones() {
	try {
		if ('undefined' == typeof g_clientZoneList) { initializeClientZones(); }
		
		g_clientZonesOnPage = [];
		var $clientZones = $(".clientZone");

		// If there aren't any client zone dom elements on the page, bail.
		if ($clientZones.length == 0) { return; }

		// Build client zones array
		$clientZones.each(function setupEachClientZone() { g_clientZonesOnPage.push(new clientZone($(this))); });

		// Set up each client zone on the page
		for ( var i = 0; i < g_clientZonesOnPage.length; i++) {
			for ( var j = 0; j < g_clientZoneList.length; j++) {
				if ("file://" + DIR_ROOT + "/data/" + g_clientZonesOnPage[i].name != g_clientZoneList[j].name) { continue; }

				var thisImage = new Image();
				thisImage.src = g_clientZoneList[j].image.src;

				// Only add this image if the file actually exists
				if (file_exists(g_clientZoneList[j].image.src.substring(7)) == "1")
					g_clientZonesOnPage[i].items.push(thisImage);
			}
			g_clientZonesOnPage[i].init();
		}
	} catch (e) {
		popError(e);
	}
}

// Find out if client-zone has been filled correctly.
function isClientZoneFilled(zone) {
	// Declare function variables.
	var listOfClientZonesOnPage = g_clientZonesOnPage,
		numberOfClientZonesOnPage = listOfClientZonesOnPage.length,
		doesClientZoneHaveImg = false,
		ix = 0,
		iix = 0;
	// If client-zone name is not set, fill it with default.
	zone = zone ? zone : 'featured_n5i';
	// Continue if there are client-zones on this page.
	if (0 < numberOfClientZonesOnPage) {
		// Outer loop.
		for (ix = 0; ix < numberOfClientZonesOnPage; ix++) {
			// Declare outer loop variables.
			var singleClientZone = listOfClientZonesOnPage[ix],
				listOfSingleClientZoneItems = singleClientZone.items,
				numberOfSingleClientZoneItems = listOfSingleClientZoneItems.length;
			// Is the expected client-zone shown on this page?
			if (zone == singleClientZone.name) {
				// Inner loop.
				for (iix = 0; iix < numberOfSingleClientZoneItems; iix++) {
					// Declare inner loop variable.
					var clientZoneItem = listOfSingleClientZoneItems[iix];
					if ('object' == typeof clientZoneItem && 'IMG' == clientZoneItem.tagName) {
						// Image was found in this client zone; exit.
						doesClientZoneHaveImg = true;
						// Exit loop if an image is found for this zone.
						break;
					}
				}
				// Exit inner loop.
				if (doesClientZoneHaveImg) {
					// Exit loop if an image is found inside this client-zone.
					break;
				}
				// Exit outer loop.
			}
		}
	}
	return doesClientZoneHaveImg;
};

function clientZone(me) {
	this.$me = me;
	this.name = me.attr("id");
	this.items = [];
	this.counter = 0;
	this.doFade = (this.$me.attr("fade") == "yes");
	this.random = (this.$me.attr("random") == "yes");
	this.loopTimeout = null;
	this.loopMS = 20 * 1000;
}

clientZone.prototype.init = function clientZoneInit() {
	var _this = this;

	clearTimeout(this.loopTimeout);

	// Immediately return (and don't show) if this zone has no images.
	if (this.items.length == 0) return;

	this.setImage();

	this.$me.show().find("img").live("mousedown", function() { return (false); });

	// If this client zone has one image, return and don't start rotation.
	if (this.items.length == 1) return;

	// Begin rotation
	this.loopTimeout = setTimeout(function loopClientZoneInit() { _this.update(); }, this.loopMS);
};

// Update client zone image bg. MP
clientZone.prototype.update = function clientZoneUpdate() {
	var _this = this;

	// Guilty until proven innocent
	var rogueTimeout = true;
	g_clientZonesOnPage.forEach(function(element, index, array) {
		if (element.items == _this.items) rogueTimeout = false;
	});
		
	if (rogueTimeout) {
		clearTimeout(this.loopTimeout);
		return;
	}

	this.counter = (this.counter < this.items.length-1) ? (this.counter + 1) : 0;

    // Set Attract screen / Client zone background.
	if(this.doFade) { 
		this.fadeImage(); 
	} else { 
		this.setImage();
	}

	if (viewManager && viewManager.activeObject && viewManager.activeObject.name == "attract") {
		$("#sunburst").css({"visibility" : "visible"});
	}

	clearTimeout(this.loopTimeout);

	this.loopTimeout = setTimeout(function loopClientZoneUpdate() {  _this.update(); }, this.loopMS);
};

// If there is just one image for this zone, set it as the background. Otherwise set and rotate.
// Transition from one client zone image background to another. MP
clientZone.prototype.swapImage = function clientZoneSwapImage(src) {
	this.$me.css({
		"background-image" : "url(" + src + ")",
		"background-position" : "50% 50%",
		"-webkit-transition" : ((this.doFade) ? "background 0.5s linear" : "")
	});
};

// Use swapImage to set client zone image background. MP
clientZone.prototype.setImage = function clientZoneSetImage() {
	var i = ((this.random)?Math.floor(Math.random()*(this.items.length)):this.counter);
	this.swapImage(this.items[i].src);
};

// Use swapImage to set client zone image background or fade using animate. MP
clientZone.prototype.fadeImage = function clientZoneFadeImage() {
	var _this = this;

	this.swapImage(_this.items[_this.counter].src);

	setTimeout(function() {
		_this.$me.find("img").animate({"opacity":"0"}, 250, "linear", function() {
			setTimeout(function() {
				_this.$me.html("<img id='attractBackground' src='" + _this.items[img].src + "' / >");
			}, 500);
		});
	}, 1000);
};

/**
 * @namespace
 * @name RotationSchedule
 * @class Runs a callback against a strict timeout cycle.
 * @example var rotationSchedule = new RotationSchedule(this, [1,2,3], 10000, function(){});
 * @param {object} context
 * @param {array} rotationArray
 * @param {number} [rotationDelay=20000]
 * @param {function} rotationCallback */
function RotationSchedule(context, rotationArray, rotationDelay, rotationCallback) {
    this.context = context;
    this.rotationArray = [].concat(rotationArray);
    // TODO:XXX: Can be updated from the server. Currently set to 20s.
    this.rotationDelay = (rotationDelay) ? rotationDelay : 20000;
    this.rotationCallback = rotationCallback;
    this.msInDay = 86400000;
    this.currentDate = Date.now;
    this.currentDateStart = (new Date(this.currentDate())).setHours(0, 0, 0, 0);
    // Tighten 'runCycleTime' to clear up display issues.
    this.runCycleTime = 1000;

    // Reset these values each cycle.
    this.runCycle = null;
    this.msPassed = 0;
    this.cyclesPassed = 0;
    this.previousCycle = 0;
    this.rotationArrayIndex = 0;

    return(this);
}

/**
 * @memberOf RotationSchedule
 * @name startTick
 * @function
 * @instance
 * @alias rotationScheduleStartTick */
RotationSchedule.prototype.startTick = function rotationScheduleStartTick() {
    var me = this;
    // Run at 0 once, and at 1 second every time afterward.
    this.runCycle = window.setTimeout(function runWorkCycleNow() {
        me.workCycle();
    }, 0);
    // Allow this function to be chained at it's init.
    return(this);
    // Could also be called as 'rotationSchedule.workCycle()'.
};

/**
 * @memberOf RotationSchedule
 * @name tick
 * @function
 * @instance
 * @alias rotationScheduleTick */
RotationSchedule.prototype.tick = function rotationScheduleTick() {
    var me = this;
    this.runCycle = window.setTimeout(function runWorkCycle() {
        me.workCycle();
    }, me.runCycleTime);
};

/**
 * @memberOf RotationSchedule
 * @name calculate
 * @function
 * @instance
 * @alias rotationScheduleCalculate */
RotationSchedule.prototype.calculate = function rotationScheduleCalculate() {
    // Calculate the position of time within our cycle.
    this.msPassed = ((this.currentDate() - this.currentDateStart));
    this.cyclesPassed = Math.floor(this.msPassed / this.rotationDelay);
    this.rotationArrayIndex = Math.floor(this.cyclesPassed % (this.rotationArray.length));
};

/**
 * @memberOf RotationSchedule
 * @name workCycle
 * @function
 * @instance
 * @alias rotationScheduleWorkCycle */
RotationSchedule.prototype.workCycle = function rotationScheduleWorkCycle() {
    // Calculate the position of time within our cycle.
    this.calculate();
    // Make sure stack frame is clear.
    this.clear();
    // Only continue if object is visible.
    if ($('#' + this.context.name).is(':visible')) {
        // Call outer function tick recursively.
        this.tick();
    }
    // Run callback in the correct outer context/stack.
    if (this.previousCycle != this.cyclesPassed && this.rotationCallback) {
        this.rotationCallback.call(this.context);
    }
    // Set work-cycle to run only in new cycle interval.
    this.previousCycle = this.cyclesPassed;
};

/**
 * @memberOf RotationSchedule
 * @name clear
 * @function
 * @instance
 * @alias rotationScheduleClear */
RotationSchedule.prototype.clear = function rotationScheduleClear() {
    // Make sure stack frame is clear.
    if (this.runCycle) {
        window.clearInterval(this.runCycle);
        this.runCycle = null;
    }
};
