/*
 * Constants
 */

/* Open flags */
var PR_RDONLY       = 0x01;
var PR_WRONLY       = 0x02;
var PR_RDWR         = 0x04;
var PR_CREATE_FILE  = 0x08;
var PR_APPEND       = 0x10;
var PR_TRUNCATE     = 0x20;
var PR_SYNC         = 0x40;
var PR_EXCL         = 0x80;

/* File mode bits */
var PR_IRWXU        = 00700;  /* read, write, execute/search by owner */
var PR_IRUSR        = 00400;  /* read permission, owner */
var PR_IWUSR        = 00200;  /* write permission, owner */
var PR_IXUSR        = 00100;  /* execute/search permission, owner */
var PR_IRWXG        = 00070;  /* read, write, execute/search by group */
var PR_IRGRP        = 00040;  /* read permission, group */
var PR_IWGRP        = 00020;  /* write permission, group */
var PR_IXGRP        = 00010;  /* execute/search permission, group */
var PR_IRWXO        = 00007;  /* read, write, execute/search by others */
var PR_IROTH        = 00004;  /* read permission, others */
var PR_IWOTH        = 00002;  /* write permission, others */
var PR_IXOTH        = 00001;  /* execute/search permission, others */

/* File types */
var PR_NORMAL_FILE_TYPE = 0
var PR_DIRECTORY_TYPE = 1

var USERNAME = 'netpulse';


/* String whitespace trimming */

String.prototype.trim = function() {
    return this.replace(/^\s*|\s*$/g, '');
}

String.prototype.ltrim = function() {
    return this.replace(/^\s*/g, '');
}

String.prototype.rtrim = function() {
    return this.replace(/\s*$/g, '');
}


/* Pseudo-inheritance
 *
 * Use like this:
 *
 * function A() {
 *     this.names = ['A'];
 * }
 * function B() {
 *     A.apply(this);
 *     this.names.push('B');
 * }
 * B.extendss(A);
 * b = new B();
 * b.names == ['A', 'B']    // true
 *
 */

Function.prototype.extendss = function(parent) {
    var m;
    for(m in parent.prototype) {
        this.prototype[m] = parent.prototype[m];
    }
};

/*
 * Exceptions
 */

function FileDoesNotExist(filename) {
    this.filename = filename;
}
FileDoesNotExist.prototype.name = 'FileDoesNotExist';
FileDoesNotExist.prototype.toString = function() {
    return this.name + ': ' + this.filename;
}


function InvalidChannelError(channel) {
    this.channel = channel;
}
InvalidChannelError.prototype.name = 'InvalidChannelError';
InvalidChannelError.prototype.toString = function() {
    return this.name + ': ' + this.channel;
}


/*
 * Utility Functions
 */

function need_privs() {
    try {
        netscape;
    } catch(e) {
        if(isObjInst(e, ReferenceError)) {
            return false;
        }
    }
    return true;
}

function callvar(fn, args) {
    if(args.length > 10) {
        throw new TypeError(IDS_LANGUAGE.installCore.genError['tooMany']);
    }
    if(args.length == 10) {
        return fn(
          args[0], args[1], args[2], args[3], args[4],
          args[5], args[6], args[7], args[8], args[9]
        );
    } else if(args.length == 9) {
        return fn(
          args[0], args[1], args[2], args[3], args[4],
          args[5], args[6], args[7], args[8]
        );
    } else if(args.length == 8) {
        return fn(
          args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7]
        );
    } else if(args.length == 7) {
        return fn(
          args[0], args[1], args[2], args[3], args[4], args[5], args[6]);
    } else if(args.length == 6) {
        return fn(args[0], args[1], args[2], args[3], args[4], args[5]);
    } else if(args.length == 5) {
        return fn(args[0], args[1], args[2], args[3], args[4]);
    } else if(args.length == 4) {
        return fn(args[0], args[1], args[2], args[3]);
    } else if(args.length == 3) {
        return fn(args[0], args[1], args[2]);
    } else if(args.length == 2) {
        return fn(args[0], args[1]);
    } else if(args.length == 1) {
        return fn(args[0]);
    }
    return fn();
}

function runProgram(filename, args) {

    //alert("runProgram("+filename+", "+print_r(args)+")");
    if(0 == eval(npfunctions.file_exists(filename))) {
        //alert("Error: " + filename + " not found.");
    }

    var strCommand = filename;

    if(args != null)
    {
        for (var i=0; i<args.length;i++) { strCommand += " " +  args[i]; }
    }

    //alert ('strCommand: ' + strCommand);
    var ret = npfunctions.run(strCommand);
    return ret;
}

function runProgramAsync(filename, args) {

    var strCommand = filename;
    for (var i=0; i<args.length;i++)
    strCommand += " " +  args[i];

    var ret = npfunctions.runAsync(strCommand, "");
    return ret;
}

function runSudoProgram(filename, args) {
    if(args != null)
    {
        var new_args = [filename].concat(args);
        //alert ('new_args: ' + new_args);
        return runProgram('/usr/bin/sudo', new_args);
    }
    else
    {
        return runProgram('/usr/bin/sudo '+filename);
    }

}

function shell_quote(parts) {
    var quoted = "";
    var i;
    for(i = 0; i < parts.length; i++) {
        part = parts[i];
        quoted_part = quote_quotes(part);

        quoted = quoted + "'";
        quoted = quoted + quoted_part;
        quoted = quoted + "'";
        if(i < (parts.length - 1)) {
            quoted = quoted + " ";
        }
    }
    return quoted;
}

function quote_quotes(s) {
    return s.replace(/'/, "'\\''");
}

function getStatusOutput(filename, args, exception_on_failure) {
    
    var tmpFilename = createTemporaryFile();
    var cmd =
      shell_quote([filename].concat(args)) +
      ' > ' +
        shell_quote([tmpFilename]) +
      ' 2>&1 '
      ;
    var new_args = ['-c', cmd];
    var sts = runProgram('/bin/sh', new_args);
    var output = readFile(tmpFilename);
    removeFile(tmpFilename);

    if(exception_on_failure) {
        if(sts != 0) {
            throw new Error(
              'Command failed: ' + cmd + ' ' + new_args + '\n' + output.trim());
        }
    }

    return [sts, output];
}

function getSudoStatusOutput(filename, args) {
    var new_args = [filename].concat(args);
    return getStatusOutput('/usr/bin/sudo', new_args);
}

function sleep(seconds) {
    runProgram('/bin/sleep', [String(seconds)]);
}

function createTemporaryFile() {
    return npfunctions.create_temporary_file();
}

function readFile(filename) {
    return npfunctions.read_file(filename);
}

function _writeToFile(filename, content, flags, permissions) {
    if(need_privs()) {
        netscape.security.PrivilegeManager.enablePrivilege(
          "UniversalXPConnect");
    }

    if(permissions == undefined) {
        permissions = 00644;
    }

    var nsILocalFile = Components.classes["@mozilla.org/file/local;1"]
      .getService(Components.interfaces.nsILocalFile);
    nsILocalFile.initWithPath(filename);

    if(! nsILocalFile.exists()) {
        nsILocalFile.create(PR_NORMAL_FILE_TYPE, permissions);
    }

    var nsIFileOutputStream = Components
      .classes["@mozilla.org/network/file-output-stream;1"]
      .createInstance(Components.interfaces.nsIFileOutputStream);
    nsIFileOutputStream.init(
      nsILocalFile,
      flags,
      permissions,
      null
    );

    nsIFileOutputStream.write(content, content.length);
    nsIFileOutputStream.close();
}

function writeFile(filename, content, permissions) {
    return _writeToFile(
      filename,
      content,
      (PR_WRONLY | PR_TRUNCATE),
      permissions
    );
}

function appendToFile(filename, content, permissions) {
    return _writeToFile(
      filename,
      content,
      (PR_RDWR | PR_APPEND),
      permissions
    );
}

function removeFile(filename) {
    return npfunctions.remove_file(filename);
}

function createDirectory(filename, permissions) {
    if(need_privs()) {
        netscape.security.PrivilegeManager.enablePrivilege(
          "UniversalXPConnect");
    }

    if(permissions == undefined) {
        permissions = 00755;
    }

    var nsILocalFile = Components.classes["@mozilla.org/file/local;1"]
      .getService(Components.interfaces.nsILocalFile);
    nsILocalFile.initWithPath(filename);

    if(! nsILocalFile.exists()) {
        nsILocalFile.create(PR_DIRECTORY_TYPE, permissions);
    }
}

function splitPath(path) {
    var parts = path.split('/');
    var basename = parts.pop();
    var dirname = parts.join('/');
    return [dirname, basename];
}

function isObjInst(instance, cls) {
    return (
      (typeof instance == "object") &&
      (instance instanceof cls)
    );
}

/*
 * Television Control
 */


var FREQUENCIES = {
  'cable': {
    2: 55.25,
    3: 61.25,
    4: 67.25,
    5: 77.25,
    6: 83.25,
    7: 175.25,
    8: 181.25,
    9: 187.25,
    10: 193.25,
    11: 199.25,
    12: 205.25,
    13: 211.25,
    14: 121.25,
    15: 127.25,
    16: 133.25,
    17: 139.25,
    18: 145.25,
    19: 151.25,
    20: 157.25,
    21: 163.25,
    22: 169.25,
    23: 218.25,
    24: 223.25,
    25: 229.25,
    26: 235.25,
    27: 241.25,
    28: 247.25,
    29: 253.25,
    30: 259.25,
    31: 265.25,
    32: 271.25,
    33: 277.25,
    34: 283.25,
    35: 289.25,
    36: 295.25,
    37: 301.25,
    38: 307.25,
    39: 313.25,
    40: 319.25,
    41: 325.25,
    42: 331.25,
    43: 337.25,
    44: 343.25,
    45: 349.25,
    46: 355.25,
    47: 361.25,
    48: 367.25,
    49: 373.25,
    50: 379.25,
    51: 385.25,
    52: 391.25,
    53: 397.25,
    54: 403.25,
    55: 409.25,
    56: 415.25,
    57: 421.25,
    58: 427.25,
    59: 433.25,
    60: 439.25,
    61: 445.25,
    62: 451.25,
    63: 457.25,
    64: 463.25,
    65: 469.25,
    66: 475.25,
    67: 481.25,
    68: 487.25,
    69: 493.25,
    70: 499.25,
    71: 505.25,
    72: 511.25,
    73: 517.25,
    74: 523.25,
    75: 529.25,
    76: 535.25,
    77: 541.25,
    78: 547.25,
    79: 553.25,
    80: 559.25,
    81: 565.25,
    82: 571.25,
    83: 577.25,
    84: 583.25,
    85: 589.25,
    86: 595.25,
    87: 601.25,
    88: 607.25,
    89: 613.25,
    90: 619.25,
    91: 625.25,
    92: 631.25,
    93: 637.25,
    94: 643.25,
    95: 91.25,
    96: 97.25,
    97: 103.25,
    98: 109.25,
    99: 115.25,
  },
  'bcast': {
    2: 55.25,
    3: 61.25,
    4: 67.25,
    5: 77.25,
    6: 83.25,
    7: 175.25,
    8: 181.25,
    9: 187.25,
    10: 193.25,
    11: 199.25,
    12: 205.25,
    13: 211.25,
    14: 471.25,
    15: 477.25,
    16: 483.25,
    17: 489.25,
    18: 495.25,
    19: 501.25,
    20: 507.25,
    21: 513.25,
    22: 519.25,
    23: 525.25,
    24: 531.25,
    25: 537.25,
    26: 543.25,
    27: 549.25,
    28: 555.25,
    29: 561.25,
    30: 567.25,
    31: 573.25,
    32: 579.25,
    33: 585.25,
    34: 591.25,
    35: 597.25,
    36: 603.25,
    37: 609.25,
    38: 615.25,
    39: 621.25,
    40: 627.25,
    41: 633.25,
    42: 639.25,
    43: 645.25,
    44: 651.25,
    45: 657.25,
    46: 663.25,
    47: 669.25,
    48: 675.25,
    49: 681.25,
    50: 687.25,
    51: 693.25,
    52: 699.25,
    53: 705.25,
    54: 711.25,
    55: 717.25,
    56: 723.25,
    57: 729.25,
    58: 735.25,
    59: 741.25,
    60: 747.25,
    61: 753.25,
    62: 759.25,
    63: 765.25,
    64: 771.25,
    65: 777.25,
    66: 783.25,
    67: 789.25,
    68: 795.25,
    69: 801.25,
    70: 807.25,
    71: 813.25,
    72: 819.25,
    73: 825.25,
    74: 831.25,
    75: 837.25,
    76: 843.25,
    77: 849.25,
    78: 855.25,
    79: 861.25,
    80: 867.25,
    81: 873.25,
    82: 879.25,
    83: 885.25,
  }
}

function lookup_frequency(signal_type, channel) {
    return FREQUENCIES[signal_type][channel];
}

var CROP_PARAMETERS = {
  '320x240': '300x224+10+8',
  '640x480': '600x448+20+16',
  '768x480': '714x448+27+16',
  '800x600': '750x560+25+20',
}

var TV_RESOLUTION = [768, 480];
var TV_CROP = CROP_PARAMETERS[TV_RESOLUTION[0] + 'x' + TV_RESOLUTION[1]];
var TV_TUNER_CONTROLS = 'saturation=65535,agc_crush=1,chroma_agc=1,full_luma_range=1';

function _MediaPlayer(w, h, initial, autoplay) {
    var i;

    if(! w) { w = 800; }
    if(! h) { h = 600; }

    this._w = w;
    this._h = h;

    this.element = document.createElement('embed');
    this.element.setAttribute('type', 'application/x-vlc-plugin');
    this.element.setAttribute('version', 'VideoLAN.VLCPlugin.2');

    this.element.setAttribute('width', this._w);
    this.element.setAttribute('height', this._h);

    this.element.setAttribute('aspect-ratio', this._w + ':' + this._h);

    // Force use of "x11" vout to work around bugs in VIA XVideo
    // implementation:
    this.element.setAttribute('vout', 'x11');

    this.element.setAttribute('src', initial);

    if(autoplay == undefined) {
        autoplay = true;
    }

    if(autoplay) {
        autoplay = 'true';
        this.was_playing = true;
    } else {
        autoplay = 'false';
    }

    this.element.setAttribute('autoplay', autoplay);
}

_MediaPlayer.prototype.getWidth = function() {
    return this._w;
}

_MediaPlayer.prototype.getHeight = function() {
    return this._h;
}

_MediaPlayer.prototype._removeElement = function() {
    if(this.element) {
        var p = this.element.parentNode;
        if(p) {
            p.removeChild(this.element);
        }
    }
}

_MediaPlayer.prototype.positionAbsolute = function(x, y) {
    this._removeElement();

    this.element.style.position = 'absolute';
    this.element.style.left = x + 'px';
    this.element.style.top = y + 'px';
    this._x = x;
    this._y = y;
    document.getElementsByTagName('BODY')[0].appendChild(this.element);
}

_MediaPlayer.prototype.positionStatic = function() {
    this._removeElement();

    this.element.style.position = 'static';
    this.element.style.left = undefined;
    this.element.style.top = undefined;
    this._x = undefined;
    this._y = undefined;

    return this.element;
}

_MediaPlayer.prototype.getX = function() {
    return this._x;
}

_MediaPlayer.prototype.getY = function() {
    return this._y;
}

_MediaPlayer.prototype.destroy = function() {
    this.stop();
    this._removeElement();
}

_MediaPlayer.prototype.play = function() {
    this.element.playlist.play();
}

_MediaPlayer.prototype.stop = function() {
    this.element.playlist.stop();
}


function TVPlayer(w, h, channel, channels, signal_type, video_device) {
    if(signal_type == undefined) {
        signal_type = 'cable';
    }
    this.signal_type = signal_type;

    if(video_device == undefined) {
        video_device = DetectTvDevice();
    }
    this.video_device = video_device;

    if(channels == undefined) {
        channels = [0];
        var i;
        for(i = 3; i <= 83; i++) {
            channels.push(i);
        }
    }
    if(channels.length < 1) {
        throw new RangeError('There must be at least one channel.');
    }
    this._channels = channels;

    if(channel == undefined) {
        channel = channels[0];
    }
    var initial = this._getTvUrl(channel);
    this._channel_index = channels.indexOf(channel);

    _MediaPlayer.call(this, w, h, initial, true);

    // Crop off black border
    this.element.setAttribute('crop', TV_CROP);

    // Enable deinterlace filter for TV
    this.element.setAttribute('vout_filter', 'deinterlace');
}

TVPlayer.extendss(_MediaPlayer);

TVPlayer.prototype._getTvUrl = function(channel) {
    var url = 'v4l2://' + this.video_device +
      ':controls-reset' +
      ':set-ctrls={' + TV_TUNER_CONTROLS + '}' +
      ':width=' + TV_RESOLUTION[0] +
      ':height=' + TV_RESOLUTION[1];
    if(channel == 0) {
        url = url + ':input=1';
    } else {
        url = url +
          ':input=0' +
          ':tuner-frequency=' +
          1000 * lookup_frequency(this.signal_type, channel);
    }
    return url;
}

TVPlayer.prototype.getChannels = function() {
    return this._channels;
}

TVPlayer.prototype.setChannels = function(channels) {
    this._channels = channels;

    var current_channel = this.getChannel();
    if(channels.indexOf(current_channel) >= 0) {
        this._channel_index = channels.indexOf(current_channel);
    } else {
        this.setChannelIndex(0);
    }
}

TVPlayer.prototype.getChannel = function() {
    return this._channels[this._channel_index];
}

TVPlayer.prototype.setChannel = function(value) {
    value = parseInt(value);

    if(this._channels.indexOf(value) < 0) {
        throw new InvalidChannelError(value);
    }

    var url = this._getTvUrl(value);

    this.element.playlist.add(url);
    if(! this.element.playlist.isPlaying) {
        this.play();
    }
    this.element.playlist.next();

    this._channel_index = this._channels.indexOf(value);
}

TVPlayer.prototype.getChannelIndex = function() {
    return this._channel_index;
}

TVPlayer.prototype.setChannelIndex = function(index) {
    return this.setChannel(this._channels[index]);
}

TVPlayer.prototype.channelNext = function() {
    var current_channel = this.getChannel();
    var new_channel_index;

    if(this._channel_index == (this._channels.length - 1)) {
        new_channel_index = 0;
    } else {
        new_channel_index = this._channel_index + 1;
    }

    return this.setChannelIndex(new_channel_index);
}

TVPlayer.prototype.channelPrev = function() {
    var current_channel = this.getChannel();
    var new_channel_index;

    if(this._channel_index == 0) {
        new_channel_index = this._channels.length - 1;
    } else {
        new_channel_index = this._channel_index - 1;
    }

    return this.setChannelIndex(new_channel_index);
}


function MediaPlayer(w, h, url, loop, autoplay) {
    _MediaPlayer.call(this, w, h, url, autoplay);

    if(loop) {
        loop = 'true';
    } else {
        loop = 'false';
    }
    this.element.setAttribute('loop', loop);

    this._scheduleCompletionCheck();
}

MediaPlayer.extendss(_MediaPlayer);

MediaPlayer.prototype._scheduleCompletionCheck = function() {
    var _this = this;
    setTimeout(function () { _this._checkIfComplete(); }, 500);
}

MediaPlayer.prototype._checkIfComplete = function() {
    var is_playing;
    try {
        is_playing = this.element.playlist.isPlaying;
    } catch(e) {
        is_playing = false;
    }
    if(this.was_playing) {
        if(is_playing) {
            this.was_playing = true;
            this._scheduleCompletionCheck();
        } else {
            this.was_playing = false;
            if(this && this.onMediaComplete) {
                var _this = this;
                setTimeout(function() { _this.onMediaComplete(); }, 500);
            }
        }
    }
}

MediaPlayer.prototype.setUrl = function(url) {
    this.stop();
    this.element.playlist.clear();
    this.element.playlist.add(url);
    this.play();
    this.element.playlist.next();
}

MediaPlayer.prototype.onMediaComplete = function() {
}

MediaPlayer.prototype.stop = function() {
    _MediaPlayer.prototype.stop.apply(this);
    this._checkIfComplete();
}

/*
 * Audio Sub-System
 */

var _volume;
var _muted;

function _initializeAudio() {
    //runProgram('/netpulse/bin/dsb-audio', ['initialize']);
    //_setVolume(50);
    //_muteOff();
}

_fixVolume = function(value) {
    if(value < 0) {
        value = 0;
    }
    if(value > 100) {
        value = 100;
    }
    return value;
}

function _setVolume(value) {
    //_volume = _fixVolume(value);
    //return (
    //  runProgram('/netpulse/bin/dsb-audio', ['set-volume', String(_volume)]) == 0);
}

function _getVolume() {
    return _volume;
}

function _getRealVolume() {
    //var status_output = getStatusOutput('scripts/dsb-audio', ['get-volume']);
    //return parseInt(status_output[1].trim());
}

function _muteOff() {
    //_muted = false;
    //return (runProgram('/netpulse/bin/dsb-audio', ['mute-off']) == 0);
}

function _muteOn() {
    //_muted = true;
    //return (runProgram('/netpulse/bin/dsb-audio', ['mute-on']) == 0);
}

function _isMuted() {
    return _muted;
}

function _toggleMuted() {
    if(_isMuted()) {
        return _muteOff();
    }
    return _muteOn();
}

/*** Netpulse API ***/

function GetVolume() {
    return _getVolume();
}

function GetRealVolume() {
    return _getRealVolume();
}

function SetVolume(value) {
    return _setVolume(value);
}

function VolumeUp(value) {
    return _setVolume(_getVolume() + value);
}

function VolumeDown(value) {
    return _setVolume(_getVolume() - value);
}

function IsMuted() {
    return _isMuted();
}

function ToggleMute() {
    return _toggleMuted();
}


/*
 * Networking
 */

function GetComputerName() {
    return configObj.getMid();
}

function GetHostname() {
    var retval;

    try {
        retval = request_txt_document("/etc/hostname");
    } finally {
        // do nothing
    }

    return retval;
}

function SetComputerName(name) {
    var retval;

    try {
        retval = runSudoProgram('/netpulse/bin/n5i-net', ['set-computer-name', name]);
    } finally {
        // do nothing
    }

    return retval;
}

function GetWirelessMacAddress() {
    var retval;

    try {
        retval = runProgram('/netpulse/bin/n5i-net', ['get-npwlan-mac-address']);
    } finally {
        // do nothing
    }

    return retval;
}

function GetEth0MacAddress() {
    var retval;

    try {
        retval = runProgram('/netpulse/bin/n5i-net', ['get-eth0-mac-address']);
    } finally {
        // do nothing
    }

    return retval;
}

function GetPrimaryIpAddress() {
    var retval;

    try {
        retval = runProgram('/netpulse/bin/n5i-net', ['get-primary-ip-address']);
    } finally {
        // do nothing
    }

    return retval;
}

function GetGatewayIpAddress()
{
    try {
    	//runProgram('/netpulse/bin/n5i-net', ['get-gateway-ip-address']).trim();
 	     return runProgram('/netpulse/bin/n5i-net', ['get-gateway-ip-address']).trim();
    } catch(e) {popError(e);}
}


function PullCert(statusCallback) {
    var retval;

    try {
        retval = npfunctions.runAsync('/netpulse/bin/n5i-sys pullcert', statusCallback);
    } finally {
        // do nothing
    }

    return retval;
}

function BringUpWireless(statusCallback) {
    var retval;
    try {
        retval = runSudoProgram('/netpulse/bin/n5i-net', ['stop']);
		retval = npfunctions.runAsyncMuxed('sudo /netpulse/bin/n5i-net start', statusCallback);
    } finally {
        // do nothing
    }
    return retval;
}

function Scan(type, statusCallback)
{
    try
    {

        var retval = false;

        set_television_tuner_type(String(type));
        var countryCode = machineObj.getLocationCountry();

        switch(type)
        {
            case 'cable_standard':
            case 'cable_irc':
            case 'cable_hrc':
            case 'analog_digital_cable':
                retval = ScanCable(statusCallback);
                break;

            case 'digital_cable':
                removeFile('/netpulse/etc/channels.conf');
                retval = npfunctions.runAsync('/netpulse/bin/scan-cable-limit.sh cable ' + countryCode, statusCallback);
                break;

            case 'digital_ota':
                removeFile('/netpulse/etc/channels.conf');
                retval = npfunctions.runAsync('/netpulse/bin/scan-cable-limit.sh ota ' + countryCode, statusCallback);
                break;

            default:
                alert('Unable to scan '+ type);
                break;
        }

        return retval;

    } catch(e) {alert(e);}
}

function ScanCable(statusCallback)
{
    try
    {
        removeFile('/netpulse/etc/stationlist.xml');
        runProgram('cp -f /netpulse/etc/stationlist.xml.default /netpulse/etc/stationlist.xml', '');

        setTimeout(
            function()
            {
                npfunctions.runAsync('/netpulse/bin/tvtime-command CHANNEL_SCAN', statusCallback);
            }
        , 4000);

        return true;
    }
    catch(e) {alert("ScanCable() - "+e);}
}

function uploadTVTunerScanFiles()
{
	var myGatewayIP = GetGatewayIpAddress();
	var myServerURL = machineObj.getServerLocation();

    try {
    	minimizeTVPlayer();
    	$("#pleaseWaitDialog").html('<p>'+IDS_LANGUAGE.install['pleaseWaitMsg']+'</p>');//<img src="images/8-1c.gif" />
    	$('#pleaseWaitDialog').dialog('open');
	
	setTimeout(function(){
      saveSetup();
      runProgram('/netpulse/bin/n5i-sys', ['upload-tv-scan', myGatewayIP, myServerURL]).trim();
	  $("#pleaseWaitDialog").dialog('close');
	  var diagMsg = '';
	  if (IDS_LANGUAGE.tvConfig)
		  diagMsg = IDS_LANGUAGE.tvConfig['upload'];
	  else
		  diagMsg = IDS_LANGUAGE.install.tvConfig['upload'];
	  var okTransString = ((IDS_LANGUAGE.installCore['ok']) ? IDS_LANGUAGE.installCore['ok'] : "ok");
	  $('#upDownDialog').dialog({
			'buttons' : [{
				text: okTransString, 
				click: function() { $(this).dialog("close"); }
			}],
			'close' : function(event, ui) { 
				// Let jQuery finish closing the dialog box first
				setTimeout(function() { maximizeTVPlayer(); }, 20); 
			}
		});
      $('#upDownDialog').html(diagMsg);
      $('#upDownDialog').dialog("open");
	}, 500 );  //mess around with the time
	
    } catch(e) {g_UI.showAlert("Error - "+e);}
}

function downloadTVTunerScanFiles()
{	
    var myGatewayIP = GetGatewayIpAddress();
    var myServerURL = machineObj.getServerLocation();

    try {
    	$("#pleaseWaitDialog").html('<p>'+IDS_LANGUAGE.install['pleaseWaitMsg']+'</p>');//<img src="images/8-1c.gif" />
    	$('#pleaseWaitDialog').dialog('open');
    	
	setTimeout(function(){
	    runProgram('/netpulse/bin/n5i-sys', ['download-tv-scan', myGatewayIP, myServerURL]).trim();
	    $("#pleaseWaitDialog").dialog('close');
		  var diagMsg = '';
		  if (IDS_LANGUAGE.tvConfig)
			  diagMsg = IDS_LANGUAGE.tvConfig['download'];
		  else
			  diagMsg = IDS_LANGUAGE.install.tvConfig['download'];
		var okTransString = ((IDS_LANGUAGE.installCore['ok']) ? IDS_LANGUAGE.installCore['ok'] : "ok");
	    $('#upDownDialog').dialog({
			 'buttons' : [{
				 text: okTransString, 
				 click: function() { $(this).dialog("close"); }
			 }]
		});
	    $('#upDownDialog').html(diagMsg);
	    $('#upDownDialog').dialog("open");
	    tvStep("scanComplete");
	}, 500 );  //mess around with the time

    } catch(e) {g_UI.showAlert(IDS_LANGUAGE.installCore.genError['error'] +e);}
}


/*
 * System
 */

function MakeDiskWritable() {    
    return true;
    //return (runSudoProgram('/netpulse/bin/dsb-disk', ['make-disk-writable']) == 0);
}

function MakeDiskUnwritable() {
    return true;
    //return (runSudoProgram('/netpulse/bin/dsb-disk', ['make-disk-unwritable']) == 0);
}

function DiskIsWritable() {
    return true;
    //return (runSudoProgram('/netpulse/bin/dsb-disk', ['disk-is-writable']) == 0);
}

function HasTvDevice() {
    return eval(npfunctions.has_television_tuner());
}

function DetectTvDevice() {
//     alert("Fix Me: DetectTvDevice()");
    //var status_output = getStatusOutput(
    //  '/netpulse/bin/n51-sys', ['detect-tv-device']);
    //return status_output[1].trim();
}

function GetPackageVersion(package_name) {
    //var status_output = getStatusOutput(
    //  '/netpulse/bin/dsb-updates', ['get-package-version', package_name], true);
    //return status_output[1].trim();

    return "TBD";
}

function InstallUpdates(delay) {
    alert(IDS_LANGUAGE.installCore['installUpdates'] + "("+delay+")");
    if(delay == undefined) {
        delay = true;
    }

//    var args = ['run', '--no-stop-app'];
//    if(! delay) {
//        args.push('--no-delay');
//    }
//    return (getSudoStatusOutput('/netpulse/bin/dsb-updates', args) == 0);
    return "";
}


/*
 * Network Configuration
 */

function SaveNetworkConfiguration(params) {
    //alert("SaveNetworkConfiguration("+print_r(params)+") : ");
    var args = ['set-network-configuration'];
    var arg, param, value, retval;

    for(param in params) {
        value = params[param];
        if(value) {
            arg = param + '=' + value;
            args.push(arg);
        }
    }

    //MakeDiskWriteable();
    try {
        retval = (runSudoProgram('/netpulse/bin/n5i-net', args) == 0);
    } finally {
        //MakeDiskUnwritable();
    }
    return retval;
}

function HasWireless() {
    return eval(runProgram('/netpulse/bin/n5i-net', ['has-wireless']));
}

function StartNetworking() {
    return (runSudoProgram('/netpulse/bin/n5i-net', ['start']) == 0);
}

function StopNetworking() {
    return (runSudoProgram('/netpulse/bin/n5i-net', ['stop']) == 0);
}

function RestartNetworking() {
    return (runSudoProgram('/netpulse/bin/n5i-net', ['restart']) == 0);
}

function TestNetwork() {
    return (runProgram('/netpulse/bin/n5i-net', ['network-is-up']) == 0);
}

function TestNetworkEx() {
    var bRet = false;
    $.ajax({
        type:"GET",
        url: "http://www.netpulse.com",
        dataType: "html",
        async:false,
        timeout:2000,
        success:function(responseXML){
            bRet = true;
        },
        error:function() {
            bRet = false;
        }
    });
    
    return bRet;
}


/*
 * Display Configuration
 */

function SaveDisplayConfiguration(params) {
    alert("fix me : SaveDisplayConfiguration()");
    var retval;
    var args = [
      'set-xorg-configuration',
      'resolution=' + params['resolution'],
      'rotation=' + (params['rotation'] ? 'on' : 'off'),
      'secondary_device=' + params['secondary_output'],
    ];

    //MakeDiskWritable();
    try {
        retval = (runProgram('/netpulse/bin/n5i-sys', args) == 0);
    } finally {
        //MakeDiskUnwritable();
    }
    return retval;
}


/*
 * Client Session
 */

function StopApplication() {
    alert("StopApplication()");
    //return (runProgram('/netpulse/bin/dsb-client', ['stop-app']) == 0);
}


/*
 * Initialization
 */

function InitializePlatform() {
    _initializeAudio();
}


/* Some auxillary functions: */
function request_txt_document(url, method, body)
{
    return $.ajax({
        type:"GET",
        url: url,
        dataType: "html",
        async:false,
        timeout:5000}).responseText;
}


function request_xml_document(url, method, body)
{
    var retObj = null;
    
    $.ajax({
        type:"GET",
        url: url,
        dataType: "xml",
        async:false,
        timeout:5000,
        success:function(responseXML){
            retObj = responseXML
        },
        error:function(XMLHttpRequest, textStatus, errorThrown) {
            alert('Error: ['+textStatus+'] errorThrown: ' + errorThrown + ' for: ' + url);
        },
    });

    return retObj;
}

function parse_xml_document(urlString) {
    return request_xml_document(urlString);
}

function serialize_xml_document(doc) {
    alert("serialize_xml_document()!!!!");
    if(need_privs()) {
        netscape.security.PrivilegeManager.enablePrivilege(
          "UniversalXPConnect");
    }

    var serializer = Components
      .classes['@mozilla.org/xmlextras/xmlserializer;1'].
      createInstance(Components.interfaces.nsIDOMSerializer);

    return serializer.serializeToString(doc.documentElement);
}

function xpath_eval(doc, expr) {
    alert("xpath_eval() OBSOLETE");
//     if(need_privs()) {
//         netscape.security.PrivilegeManager.enablePrivilege(
//           "UniversalXPConnect");
//     }
// 
//     doc.QueryInterface(Components.interfaces.nsIDOMXPathEvaluator);
//     var result = doc.evaluate(
//       expr,
//       doc.documentElement,
//       null,
//       Components.interfaces.nsIDOMXPathResult.ANY_TYPE,
//       null
//     );
//     result.QueryInterface(Components.interfaces.nsIDOMXPathResult);
//     return result;
}

function attr_value(attr, set_to) {
    if(set_to == undefined) {
        return attr.nodeValue;
    }
    attr.nodeValue = String(set_to);
}

function reboot()
{
   try {
        retval = runSudoProgram('/sbin/reboot', null);
    } finally {
        // do nothing
    }
}

function RestartX()
{
    try {
        retval = runProgram('/bin/sh /netpulse/bin/n5i-sys', ['restart-x']);
    } finally {
        // do nothing
    }
}

function getInstallFileContent()
{
    return getFileContent(DIR_ROOT + "/data/install");
}

function getFileContent(file)
{
    var retval;

    try {
        retval = request_txt_document(file);
    } finally {
        // do nothing
    }

    return retval;
}

function getCurrentChannelName()
{
    try {
        var cName = channelTitle();
        var temp = Array();
        temp = cName.split(':');
        cName = temp[0];
    }
    catch(e) {
        alert('getCurrentChannelName() exception: ' + e.message);
    }
    return cName;
}
