/*=====================================================================
	Branded Channel Script (CORE) | views/brandedChannel/js.js
=====================================================================*/

/* BRANDED CHANNEL CONSTRUCTOR ---------------------------- */

// Branded Content JS Constructor
function brandedContentObj(name, type, vm) {
	var me = this;

	me.name = name;
	me.type = type;
	me.vm = vm;
	// ToDo: Use QT Bridge machineObj instead of npdata.machine_xml
	me.hasGateway = true; //eval(npdata.machine_xml.has_gateway);
	me.gatewayIP = systemObj.getGatewayIPAddress();
	me.volume = machineObj.getVolumeLevel();
	me.nowPlaying = 0;
	me.timeStamp;
	me.adPlayed = false;
	me.videoData = {};
	me.itemsPerPage = ((isPctv) ? 3 : 4);
	me.contentType = 'branded_carousel';

    return me;
}

// Prototype of Branded Content JS, inherits
// basic view functions init(), preload(), load(), show(), hide().
brandedContentObj.prototype = new app;

/* END BRANDED CHANNEL CONSTRUCTOR ------------------------ */


//----------------------------------------------
//Shows the Branded content view with its css/html/javascript elements
brandedContentObj.prototype.show = function coreBrandedContentObjShow() {
	var me = this, returnValue = false;

	try {
		me.adPlayed = false;
		// Hook-in functions to run first.
		me.preShowHook();
		//
		me.showAds();
		// Find DOM elements.
		me.bindHTMLElements();
		// Set default volume level.
		me.initVolume();
		// Fill text place-holders using the IDS_LANGUAGE object.
		me.htmlInternationalString();
		// Create the menuData object.
		me.setupMenuData();
		// Create the video carousel for videos to be played.
		me.buildMenuData();
		// Bind mouse-down and click events.
		me.attachEvents();
		// Hook-in functions to run last.
		me.postShowHook();
		returnValue = true;
	} catch(e) {
		popError(e);
		returnValue = false;
	}

	return(returnValue);
};

//----------------------------------------------
// Hook-in functions to run first.
brandedContentObj.prototype.preShowHook = function coreBrandedContentObjPreShowHook() {
	var me = this;
	return me;
};

//----------------------------------------------
// Hook-in functions to run last.
brandedContentObj.prototype.postShowHook = function coreBrandedContentObjPostShowHook() {
	var me = this;
	return me;
};

//----------------------------------------------
// Sets default volume on start.
brandedContentObj.prototype.initVolume = function coreBrandedContentObjInitVolume() {
	var me = this;

	try {
		//ipod.stopAudio();
		setTunerInput("2");
		setVolume("ondemand", ("" + me.volume));
		enableAudio();
		me.$volLevel.html(me.volume);
	} catch(e) {
		popError(e);
	}
	
	return me;
};

//----------------------------------------------
// Handles the changing of volume (max vol, min vol).
brandedContentObj.prototype.volumeChange = function coreBrandedContentObjVolumeChange(change, amount) {
	var me = this, volume = (1 * me.volume)-0;

	amount = amount || 5;

	switch(change.toLowerCase()) {
		case "up": me.volume = (volume + amount); break;
		default: me.volume = (volume - amount); break;
	}

	if (100 <= me.volume) me.volume = 100;
	if (0 >= me.volume) me.volume = 0;
	
	// Always do 'ondemand' here as it just means not TV.
	setVolume("ondemand", String(me.volume));

	me.$volLevel.html(me.volume);
	
	if (me.unmuteVolTimeout) clearTimeout(me.unmuteVolTimeout);
	if ("1" == isMuted()) {
		me.unmuteVolTimeout = setTimeout(function runUnmuteVolTimeout() { unmuteVol(); }, 200);
	}
	
	return me;
};

//----------------------------------------------
// Increase the volume
brandedContentObj.prototype.volumeUp = function coreBrandedContentObjVolumeUp() {
	var me = this;

	me.volumeChange("up");

	return me;
};

//----------------------------------------------
// Decrease Volume
brandedContentObj.prototype.volumeDn = function coreBrandedContentObjVolumeDn() {
	var me = this;

	me.volumeChange("dn");

	return me;
};

//----------------------------------------------
// Binds all the variables to their css elements
brandedContentObj.prototype.bindHTMLElements = function coreBrandedContentObjBindHTMLElements() {
	var me = this;

	me.$mainContainer = $("#brandedChannelView");
	me.$brandedChannelPage = me.$mainContainer.find("#brandedChannelPage");
	me.$header = me.$mainContainer.find("#header h1");
	me.$volInfo = me.$mainContainer.find("#volumeText");
	me.$volLevel = me.$volInfo.find("span.level");
	me.$volLabel = me.$volInfo.find("strong.label");
	me.$volUp = me.$mainContainer.find("#volUpButton");
	me.$volDown = me.$mainContainer.find("#volDnButton");
	me.$backButton = me.$mainContainer.find("#carouselBack");
	me.$nextButton = me.$mainContainer.find("#carouselNext");
	me.$menuButton = me.$mainContainer.find("#backToMenuButton");
	me.$menuButtonLabel = me.$menuButton.find("span");
	me.$playAgainButton = me.$mainContainer.find('#playAgain');
	me.$videoCarousel = me.$mainContainer.find("#videoCarousel");
	me.$carouselContainer = me.$mainContainer.find("#carouselContainer");
	me.$contentCarousel = me.$mainContainer.find("#contentCarousel");
	me.$videoPlayer = me.$mainContainer.find("#videoContainer");
	me.videoPlayer = document.getElementById("videoContainer");
	
	return me;
};

//----------------------------------------------
// Fill text place-holders using the IDS_LANGUAGE object.
brandedContentObj.prototype.htmlInternationalString = function coreBrandedContentObjHtmlInternationalString() {
	var me = this;

	me.$menuButtonLabel.html(IDS_LANGUAGE.watch.backToMenu);
	me.$volLabel.html("volume");

	return me;
};

// ----------------------------------------------
// Attach all events to their HTML elements
brandedContentObj.prototype.attachEvents = function coreBrandedContentObjAttachEvents() {
	var me = this;

	me.$menuButton.bind("mousedown." + me.name, function() {
		me.saveImpression();
		if (me.vm.history.length > 1)
			me.vm.showPreviousView();
		else
			me.vm.show("mainMenu");
	});
	me.$volUp.bind("mousedown." + me.name, function() {
		me.volumeUp();
	});
	me.$volDown.bind("mousedown." + me.name, function() {
		me.volumeDn();
	});
	me.$playAgainButton.bind("mousedown." + me.name, function() {
		me.playVideo(0);
	});

	return me;
};

//----------------------------------------------
// Create the video-data for carousel.
brandedContentObj.prototype.setupMenuData = function coreBrandedContentObjSetupMenuData() {
	var me = this, videoDataItems = adMan.showAd(me.contentType);

	me.videoData = {
		// Retrieve content for the carousel. HTML is generated by this view, not the adManager
		items : videoDataItems,
		levels : [ {
			attachEvents : function coreBrandedContentMenuAttachEvents(obj, $container) {
				$container.find("li").each(function() {
					//
					$(this).bind("mousedown", function() {
						// Play chosen video
						var val = $(this).find(".item_id").val();
						me.saveImpression();
						me.playVideo(val);
						// Highlight chosen video
						me.highlightVideo(val);
					});
				});
			},
			drawItem : function coreBrandedContentMenuDrawItem(data) {
				var itemHtml = "<div style='background-image:url("
					+ data.thumbnail
					+ ");'><!--//--></div><h3 class='title'><span>"
					+ data.title
					+ "<span></h3>";
				return (itemHtml);
			},
			itemsPerPage : me.itemsPerPage
		} ]
	};

	return me;
};

//----------------------------------------------
// Highlight the selected carousel object, normally an 'li'
brandedContentObj.prototype.highlightVideo = function coreBrandedContentObjHighlightVideo(id) {
	var me = this;

	//  Highlight chosen video
	me.$contentCarousel.find("li").each(function() {
		var $item = $(this), $div = $item.find("div");
		if (id == $item.find(".item_id").val()) {
			$item.addClass("selected");
			$div.addClass("selected");
		} else {
			$item.removeClass("selected");
			$div.removeClass("selected");
		}
	});

	return me;
};

//----------------------------------------------
//
brandedContentObj.prototype.showAds = function coreBrandedContentObjShowAds() {
	// Load content from adManager
	// Write the html for the background
	adMan.showAd('branded_bg');
};

//
brandedContentObj.prototype.hideAds = function coreBrandedContentObjHideAds() {
	adMan.hideAd('branded_bg');
};

//----------------------------------------------
// Uses data and builds the correct menu for branded page.
brandedContentObj.prototype.buildMenuData = function coreBrandedContentObjBuildMenuData() {
	var me = this;

	if (1 == me.videoData.items.length) {
		me.$brandedChannelPage.addClass("single");
		// Automatically play the only video.  Wrap in 100ms timeout to give the Fantaay units a chance to finish drawing the view
		if (me.playVideoTimeout) clearTimeout(me.playVideoTimeout);
		me.playVideoTimeout = setTimeout(function runPlayVideoTimeout() { me.playVideo(0); }, 100);
	} else {
		this.createVideoCarousel();
	}

	return me;
};

// ----------------------------------------------
// Create the video carousel for videos to be played.
brandedContentObj.prototype.createVideoCarousel = function coreBrandedContentObjCreateVideoCarousel() {
	var me = this;

	try {
		me.videoCarousel = new slidingBox(
			me.videoData,
			me.$carouselContainer,
			me.$contentCarousel,
			me.$backButton,
			me.$nextButton,
			1,
			200
		);

		// TODO:XXX:PT:https://www.pivotaltracker.com/story/show/35568075
		me.$contentCarousel.find("li").each(function(index, domElm) {
			var dataId = me.videoData.items[index].campaign;
			$(this).attr({ 'rel' : dataId });
			if (dataId == me.vm.campaignId) {
				if (!me.adPlayed) {
					$(this).trigger("mousedown");
					me.adPlayed = true;
				}
			} else if (0 == ($(this).find(".item_id").val()))
				$(this).trigger("mousedown");
		});
	} catch (e) {
		popError(e);
	}

	return me;
};

//----------------------------------------------
// Function that will play the correct video for branded page
brandedContentObj.prototype.playVideo = function coreBrandedContentObjPlayVideo(id) {
	var me = this, playerString = '';

	try {
		me.nowPlaying = id;
		// If a video is currently playing, hide it. This takes 100ms.
		stopVideo(me.$videoPlayer);
		me.$videoPlayer.show().css("visibility", "visible");

		playerString = '<object type="video/mpeg" vout="yes" id="videoPlayer" data="' + me.videoData.items[id].location + '">' +
			'<param name="allowScriptAccess" value="always">' +
			'<param name="startInFullscreen" value="false">' +
			'<param name="wmode" value="transparent">' +
			'<param name="is_pauseable" value="false" />' +
			'<param name="data_type" value="ondemand" />' +
			'<param name="fullscreen_support" value="' + ((machineObj.getSlaveState() == 3)? "no":"yes") + '"/>' +
			'</object>';

		//  After 1 second, write the new player
		if (me.startVideoTimeout) clearTimeout(me.startVideoTimeout);
		me.startVideoTimeout = setTimeout(function() {
			playVideo(me.$videoPlayer, playerString);
		}, 500);

		me.timeStamp = adMan.ads[me.contentType].constructTimestamp();
		adMan.ads[me.contentType].success(adMan.findContentId(me.videoData.items[id]),me.timeStamp,0);

	} catch(e) {
		popError(e);
	}

	return me;
};

//----------------------------------------------
// Save impression for this ad unit
brandedContentObj.prototype.saveImpression = function coreBrandedContentObjSaveImpression() {
	var me = this, found = null, playTime = guiObj.getVideoTimeElapsedInSeconds();
	
	if(playTime > 0)
		found = adMan.addImpressionCarousel(me.videoData.items[me.nowPlaying], me.contentType, me.timeStamp, playTime);
	
	guiObj.resetVideoTimeElapsedInSeconds();
	return me;
};

//----------------------------------------------
// Plays next video in the brandedContent Page
brandedContentObj.prototype.nextEpisode = function coreBrandedContentObjNextEpisode() {
    // Only one video; do not loop
    var me = this;
    try {
        if ('undefined' != typeof me.vm.campaignId) {
            if (me.vm.history.length > 1) {
                return (me.vm.showPreviousView());
            }
        } else {
            me.nextVideo();
        }
    } catch (e) {
        popError(e);
    }
};
brandedContentObj.prototype.nextVideo = function coreBrandedContentObjNextVideo() {
    try {
        var me = this, nextVideoId = 0, nextPlayingPage = 0;
		me.saveImpression();
        if (1 == me.videoData.items.length) {
            me.$videoPlayer.hide();
            return me;
        }
        nextVideoId = ((0 + me.nowPlaying) >= (me.videoData.items.length - 1)) ? 0 : (1 + (1 * me.nowPlaying));
        nextPlayingPage = Math.ceil((1 + (1 * nextVideoId)) / me.itemsPerPage);
        if ((nextPlayingPage != Math.ceil((1 + (1 * me.nowPlaying)) / me.itemsPerPage))) {
            me.videoCarousel.scrollTo((nextPlayingPage - 1));
        }
        me.highlightVideo(nextVideoId);
        me.playVideo(nextVideoId);
    } catch (e) {
        popError(e);
    }
};

/*===================================================================*/
//@ sourceURL=views/brandedChannel/js.js
