/*=====================================================================
	User Information Script (CORE) | views/info/js.js
=====================================================================*/

// Constructor for the User Info Object.
function infoObj(name, type, vm) {
	var me = this;

	// App properties.
	me.name = name;
	me.type = type;
	me.vm = vm;

	// HTML/DOM elements.
	me.$container = null;
	me.$container = null;
	me.$infoHeader = null;
	me.$title = null;
	me.$infoContent = null;
	
	//user's last workout info
	me.userName = null;
	me.lastWorkoutLocation = null;
	me.lastWorkoutDate = null;
	me.lastWorkoutDistance = null;
	me.lastWorkoutCalories = null;
	me.lastWorkoutDuration = null;
	me.lastWorkoutActivity = null;

	return me;
}

// Inherit all of the page life-cycle methods.
infoObj.prototype = new app;

// Show the User Info View.
infoObj.prototype.show = function coreInfoObjShow() {
	var me = this;
	try {
		// Add functions to the beginning of the view.
		me.preShowHook();
		// Find DOM elements.
		me.bindHTMLElements();
		me.hideAllMessages();
		// Bind listeners to DOM elements.
		me.attachEvents();
		// Internationalize text.
		me.htmlInternationalString();
		// Get user data.
		me.getData();
		// Add functions to the end of the view.
		me.postShowHook();
		
	} catch (e) {
		popError(e);
		return false;
	}

	return true;
};

// Add functions to the beginning of the view.
infoObj.prototype.preShowHook = function coreInfoObjPreShowHook() {
	var me = this;
	me.resetData();
	return me;
};

infoObj.prototype.resetData = function coreInfoObjResetData(){
	var me = this;

		me.userName = null;
		me.lastWorkoutLocation = null;
		me.lastWorkoutDate = null;
		me.lastWorkoutDistance = null;
		me.lastWorkoutCalories = null;
		me.lastWorkoutDuration = null;
		me.lastWorkoutActivity = null;


	return me;
};

infoObj.prototype.fetchWorkoutHistory = function coreInfoObjFetchWorkoutHistory() {
	var me = this;

	// Helper function for jsonSuccess in fetchWorkoutHistory.
	function doAfterJson(data) {
		me.assignUserData();
		// Make sure that you display the data, only after making the server
		// call
		me.displayData();
	}

	// Main helper function for fetchWorkoutHistory.
	function jsonSuccess(data) {
		user.workoutHistory.total = data.total;
		for ( var i = 0, workoutDataLength = data.workouts.length; i < workoutDataLength; i++ ) {
			user.workoutHistory.workouts.push(data.workouts[i]);
		}
		doAfterJson(data);
	}

	if (user.workoutHistory.workouts.length == 0) {
		if (machineObj.getMachineDemoNetwork()) {
			$.getJSON(DIR_ROOT + '/www/demo/n5ihistory.json',
				{ // JSON post data.
					user_id : 'netpulse'
				},
				function(data) {
					jsonSuccess(data);
				});
		} else {
			$.getJSON(machineObj.getServerLocation() + "n5ihistory.jsp",
				{ // JSON post data.
					user_id : athleteObj.get_user_id(),
					machine_id : configObj.getMid(),
					beg : "1",
					end : "100"
				},
				function(data) {
					jsonSuccess(data);
				});
		}
	} else {
		doAfterJson();
	}
	return me;
};

// Find DOM elements.
infoObj.prototype.bindHTMLElements = function coreInfoObjBindHTMLElements() {
	var me = this;

	me.$container = $('div#infoView');
	me.$infoHeader = me.$container.find('header#infoHeader');
	me.$title = me.$infoHeader.find('h1.title');
	me.$infoContent = me.$container.find('#infoContent');
	
	me.$errorMessage = me.$infoContent.find("#errorMessage");
	me.$loginButton = me.$errorMessage.find("#loginButton");
	
	me.$userDetails = me.$infoContent.find("#userDetails");
	me.$userName = me.$userDetails.find("#userName");
	me.$clubName = me.$userDetails.find("#clubName");
	
	me.$workoutDetails = me.$infoContent.find("#workoutDetails");
	me.$lastWorkoutDate = me.$workoutDetails.find("#lastWorkoutDate");
	me.$lastWorkoutDistance = me.$workoutDetails.find("#lastWorkoutDistance");
	me.$lastWorkoutCalories = me.$workoutDetails.find("#lastWorkoutCalories");
	me.$LastWorkoutDuration = me.$workoutDetails.find("#LastWorkoutDuration");
	me.$LastWorkoutActivity = me.$workoutDetails.find("#LastWorkoutActivity");
	return me;
};

// Bind listeners to DOM elements.
infoObj.prototype.attachEvents = function coreInfoObjAttachEvents() {
	var me = this;
	return me;
};

// Internationalize text.
infoObj.prototype.htmlInternationalString = function coreInfoObjHtmlInternationalString() {
	var me = this;
	
	me.$userName.find("span:nth-child(1)").html(IDS_LANGUAGE.userProfile['userName']);
	me.$clubName.find("span:nth-child(1)").html(IDS_LANGUAGE.userProfile['clubName']);
	
	me.$lastWorkoutDate.find("span:nth-child(2)").html(IDS_LANGUAGE.userProfile['last_workout_header']);
	me.$lastWorkoutDistance.find("span:nth-child(2)").html(IDS_LANGUAGE["Distance"]);
	me.$lastWorkoutCalories.find("span:nth-child(2)").html(IDS_LANGUAGE["Calories"]);
	me.$LastWorkoutDuration.find("span:nth-child(2)").html(IDS_LANGUAGE["Duration"]);
	me.$LastWorkoutActivity.find("span:nth-child(2)").html(IDS_LANGUAGE.userProfile["activity"]);
	
	me.$errorMessage.find("span").html(IDS_LANGUAGE.userProfile['completeWorkoutToBegin']);
	
	return me;
};

infoObj.prototype.drawLoginButton = function infoObjDrawLoginButton() {
    var me = this;
    if (!userIsLoggedIn) {
        me.$loginButton = $("<button id='loginButton'><span>" + IDS_LANGUAGE["login/regString"] + "</span></button>");
        $("#errorMessage").append(me.$loginButton);
    
        if (!configObj.isInstalled() && !machineObj.getMachineDemoNetwork()) {
            me.$loginButton.bind("mousedown." + this.name, function () { 
                showModal(IDS_LANGUAGE.installer["notInstalledFeature"], 5000);
            });
        } else {
            me.$loginButton.bind("mousedown." + this.name, function () { 
                viewManager.nextView = viewManager.activeObject.name; 
                viewManager.show("login"); 
            });
        }
    }
};

infoObj.prototype.showUserInfoErrorMessage = function coreInfoObjShowErrorMessage(){
	var me = this;
	me.drawLoginButton();
	me.$workoutDetails.hide();
	me.$userDetails.hide();
	me.$errorMessage.show();
};

infoObj.prototype.hideUserInfoErrorMessage = function coreInfoObjHideErrorMessage(){
    var me = this;
    me.$errorMessage.hide();
    me.$workoutDetails.show();
    me.$userDetails.show();
};

infoObj.prototype.hideAllMessages = function coreInfoObjHideAllMessages(){
    var me = this;
    me.$errorMessage.hide();
    me.$workoutDetails.hide();
    me.$userDetails.hide();
};

// Get user data.
infoObj.prototype.getData = function coreInfoObjGetData() {
	var me = this;
	if(user.userIsLoggedIn) {
	    me.hideUserInfoErrorMessage();
		me.fetchWorkoutHistory();
	} else {
		me.showUserInfoErrorMessage();
	}
};

// Display user data.
infoObj.prototype.displayData = function coreInfoObjDisplayData() {
	var me = this;

		if(machineObj.getMachineDemoNetwork()){
			me.$userName.find("span:nth-child(2)").html(IDS_LANGUAGE["demoUser"]);
			me.$clubName.find("span:nth-child(2)").html(IDS_LANGUAGE["demoHomeClub"]);
		} else {
			me.$userName.find("span:nth-child(2)").html(me.userName);
			me.$clubName.find("span:nth-child(2)").html(me.lastWorkoutLocation);
		}

		if (user.workoutHistory.workouts.length > 0) {
			writeLog("info", "in core info display Data 'if' ");
			
			me.hideUserInfoErrorMessage();
			
			me.$lastWorkoutDate.find("span:nth-child(1)").html(me.lastWorkoutDate);
			me.$lastWorkoutDistance.find("span:nth-child(1)").html(me.lastWorkoutDistance);
			me.$lastWorkoutCalories.find("span:nth-child(1)").html(me.lastWorkoutCalories);
			me.$LastWorkoutDuration.find("span:nth-child(1)").html(me.lastWorkoutDuration);
			me.$LastWorkoutActivity.find("span:nth-child(1)").html(me.lastWorkoutActivity);
			writeLog("info", "out of core info display Data 'if' ");
		} else {
			me.showUserInfoErrorMessage();
			writeLog("info", "something is wrong with the if statement of info core displayData");
	}
	return me;
};

infoObj.prototype.assignUserData = function coreInfoObjAssignUserData() {
	var me = this, locName = machineObj.getLocationName(), workout, workoutDate;

	// check to see if firstName is set
	me.userName = (user.firstName && "null" != user.firstName) ? user.firstName : athleteObj.get_user_id();
	me.lastWorkoutLocation = ("" != locName && null != locName && "undefined" !== typeof locName) ? locName : IDS_LANGUAGE.notAvailable;

	if (0 < user.workoutHistory.workouts.length) {
		workout = user.workoutHistory.workouts[0];

		if (workout.date_time) {
			var workoutDateSplit = ((workout.date_time.split(" "))[0].split("-"));
			workoutDate = workoutDateSplit[1] + "/" + workoutDateSplit[2] + "/" + workoutDateSplit[0];
		} else {
			workoutDate = "";
		}
		me.lastWorkoutDate = workoutDate;

		me.lastWorkoutDistance = (workout.distance && workout.distance_units) ? workout.distance + " " + workout.distance_units : "0 " + IDS_LANGUAGE.miles;
		me.lastWorkoutCalories = (workout.calories) ? workout.calories : "0";
		me.lastWorkoutDuration = (workout.duration) ? convertSecToReadableTime(workout.duration) : "00:00:00";
		me.lastWorkoutActivity = (workout.type) ? workout.type : "";
	}

	return me;
};

// Add functions to the end of the view.
infoObj.prototype.postShowHook = function coreInfoObjPostShowHook() {
	var me = this;
	return me;
};

// Clean-up view and hide.
infoObj.prototype.hide = function coreInfoObjHide() {
	var me = this;
	me.resetData();
	return me;
};