/*=====================================================================
	Login Script (CORE) | views/login/js.js
=====================================================================*/

// Clear custom playlist data.
userCustomPlaylist.reset();

// Login object constructor.
function loginObj(name, type, vm) {
	var me = this;
	me.name = name;
	me.type = type;
	me.vm = vm;
	me.currentPage = 4;
	me.loginAttempts = 0;
	me.passwdImageCount = 48;
	me.imgMatrixSize = 16;
	me.rndImgList = null;
	me.userPasscode = null;
    me.userPasscodeCheck = null;
	me.selectedPasswordImage = -1;
	me.userPassword = -1;
	me.userName = null;
	me.userId = null;
	me.fbuid = null;
	me.age = 0;
	me.weight = 0;
	me.gender = 1;
	me.showTOS = '';

	// how long to delay before sending the resp message to oem
	me.loginRespDelay = 10;
	me.oem = null;
	me.useUuidOnLogin = true;

	// slidingBox
	me.hidePagination = null;
	me.pagesToDisplay = [ 'page0', 'page1', 'page2', 'page3' ];
	me.$cueContainer = null;

	// HTML
	me.$mainContainer = null;
	me.$backButton = null;
	me.$nextButton = null;
	me.$viewPort = null;
	me.$slidingViewPort = null;
	me.$loginContainer = null;
	me.$registerFb = null;
}

//Inherit basic view functions
//like init(), show(), and hide() from app.
loginObj.prototype = new app; 
loginObj.prototype.attachEvents = function coreLoginObjAttachEvents() {
	var me = this;

	// Keyboard functionality TODO: Scope problem
	me.$mainContainer.find("#userIDKeyboard").bind("mousedown." + me.name, function(e) {
		me.keyboardInput(e);
	});
	// Keyboard for Passcode
	me.$mainContainer.find("#userPasscodeKeyboard").bind("mousedown." + me.name, function(e) {
		me.userPasscodeKeyboardInput(e);
	});

	// Keyboard for Passcode Migration
	me.$mainContainer.find("#setNewUserPasscodeKeyboard").bind("mousedown." + me.name, function(e) {
		me.setNewUserPasscodeKeyboardInput(e);
	});

    // Cancel button sends cancel message to OEM
    me.$mainContainer.find("#cancel").bind("mousedown." + me.name, function() {
    	if(viewManager.nextView == null)
    		nsSend2("op=button", "id=logincnx");
    	else
    		viewManager.showNextView();
    });

	// Register button alerts disabled message
	me.$mainContainer.find("#register").bind("mousedown." + me.name, function() {
		viewManager.show('registration');
	});

    $(window).bind('resize.' + me.name, function coreLoginObjResize(e) {
        var activeObject = viewManager.activeObject,
            slidingBox = activeObject.slidingBox;
        if (activeObject && slidingBox) {
            slidingBox.resetBoxWidth(e);
        }
    });
    
	// Privacy Policy link
	me.$mainContainer.find("#privacyLink").bind("mousedown." + me.name, function() {
		$('#privacyPolicy').show();
	});

	me.$mainContainer.find("#privacyPolicy button#privacy_close").bind("mousedown." + me.name, function() {
		$('#privacyPolicy').hide();
	});

};

// Grab HTML elements for future reference
loginObj.prototype.bindHTMLElements = function coreLoginObjBindHTMLElements() {	
	this.$mainContainer = $("#loginView");
	this.$backButton = $("#backButton");
	this.$nextButton = $("#nextButton");
	this.$viewPort = $("#viewPort");
	this.$slidingViewPort = $("#slidingViewPort");
	this.$loginContainer = $("#loginContainer");
	this.$userPasscodeInput = $('#userPasscodeInput');
	this.$newUserPasscodeInput = $('#setNewUserPasscodeInput');
	this.$passwordDialog = $("#passwordDialog");
	this.$selectedImage = $("#selectedImage");
	this.$registerFb = this.$mainContainer.find("#registerFb");
	this.$pass_container = this.$mainContainer.find('#pass_container').hide();
};

loginObj.prototype.preShowHook = function coreLoginObjpreShowHook() {};

// Show the login view.
loginObj.prototype.show = function coreLoginObjShow() {
	var me = this, returnValue = false;
	try {
		// Hook-in functions to run first.
		me.preShowHook();
		// Find DOM elements.
		me.bindHTMLElements();
		// Create the menuData object.
		me.setupMenuData();
		// Build the sliding box and attach to the view-port.
		me.buildSlideBox();
		// Bind mouse-down and click events.
		me.attachEvents();
		// Fill text place-holders using the IDS_LANGUAGE object.
		me.htmlInternationalString();

		// Hook-in functions to run last.
		me.postShowHook();
		returnValue = true;
	} catch (e) {
		popError(e);
		returnValue = false;
	}
	return(returnValue);
};

loginObj.prototype.postShowHook = function coreLoginObjpostShowHook() {};

loginObj.prototype.hide = function coreLoginObjHide() {
    $(window).unbind('resize.' + this.name);
};

loginObj.prototype.setupMenuData = function coreLoginObjBuildMenuData() {
	var me = this;

	me.menuData = {
		levels : [ {
			drawItem : function loginPageDrawItem(data) {
                // Remove pages to memory before popping back into page.
                var $el = $("#" + data), $saved = $el.clone(true);
                $el.remove();
                return($saved.html());
			},
			attachEvents : function loginPageAttachEvents(obj, $container) {
				$container.find("li").each(function() {
					// even though this is an empty function it's still needed for init.
				});
			},
			scrollNext : function loginPageScrollNext(e) {
				me.next(e);
			},
			scrollBack : function loginPageScrollBack(e) {
				me.back(e);
			}
		} ],
		items : me.pagesToDisplay
	};
};

// When view is passed from registration, make sure that you don't get stuck in a loop.
loginObj.prototype.slidingBoxExit = function coreLoginObjSlidingBoxExit() {
    try {
        if (("registration" == viewManager.previousView.name) && (viewManager.originView)) {
            viewManager.show(viewManager.originView);
        } else {
            viewManager.showPreviousView();
        }
    } catch(err) { popError(err); }
};

loginObj.prototype.buildSlideBox = function coreLoginObjBuildSlideBox() {
	var me = this,
		$vp = (0 < this.$slidingViewPort.length) ? this.$slidingViewPort : this.$viewPort;

	me.slidingBox = new slidingBox(
		me.menuData,
		$vp,
		me.$loginContainer,
		me.$backButton,
		me.$nextButton,
		1,
		200,
		me.slidingBoxExit,
		me.$cueContainer,
		me.hidePagination
	);
};

loginObj.prototype.next = function coreLoginObjNext(e) {
	var me = this;
	// We (un)bind this function because a double click can
	// cause the window to go past its target.
	me.menuData.levels[0].scrollNext = function(){};
	switch(this.slidingBox.currentPage) {
		case 0:
			// is this the real user?
			me.validateUserId($("#id_login_userid").val());
			break;
		case 1:
			me.validateUserPasscode($("#userPasscodeInput").val());
			break;
		case 2:
			me.validatePassword();
			break;
		case 3:
			me.setNewUserPasscode($("#setNewUserPasscodeInput").val());
			break;
		default:
			break;
	}
	me.clearTimeout(me.toAttachScrollNext);
	me.toAttachScrollNext = me.setTimeout(function runAttachScroll() {
		me.menuData.levels[0].scrollNext = function loginObjMenuDataScrollNext(e) {
			me.next(e);
		};
	}, 1000);
};

loginObj.prototype.back = function coreLoginObjBack(e) {
	var me = this;
	// We (un)bind this function because a double click can
	// cause the window to go past its target.
	me.menuData.levels[0].scrollBack = function(){};
	switch(this.slidingBox.currentPage) {
		case 0:
			//return to last page			
			me.slidingBoxExit();
			break;
		case 1:
			//scroll back to login home
			this.slidingBox.scrollBack();
            //this.resetValues();
			break;
		case 2:
			//jump to login home
			this.slidingBox.scrollJump(0);
            this.resetValues();
			break;
		case 3:
			//scroll back to password selection
			if (this.previousSelection != null) {
				$(this.previousSelection).removeClass('selected');
				this.slidingBox.scrollBack();
				this.loadRndPasswdImgs();
				this.drawRndPasswdImgs();
			}
			else
				this.slidingBox.scrollJump(0);
			break;
		default:
			break;
	}
	this.showHideNextBackButtons();
	me.clearTimeout(me.toAttachScrollBack);
	me.toAttachScrollBack = me.setTimeout(function runAttachScroll() {
		me.menuData.levels[0].scrollBack = function loginObjMenuDataScrollBack(e) {
			me.back(e);
		};
	}, 1000);
};

loginObj.prototype.showHideNextBackButtons = function coreLoginObjShowHideNextBackButtons() {
	this.$nextButton.removeClass("hidden");
	this.$backButton.removeClass("hidden");
	switch(this.slidingBox.currentPage) {
	case 4:
		this.$nextButton.hide();
		this.$backButton.hide();
		break;
	default:
		this.$nextButton.show();
		this.$backButton.show();
		break;
	}
};

loginObj.prototype.displayWelcomeMsg = function coreLoginObjDisplayWelcomeMsg() {
	var me = this,
        name = ((user.firstName && user.firstName != "null") ? user.firstName : me.userId),
		msg = IDS_LANGUAGE.login.welcome + name + IDS_LANGUAGE.login.sucessfullLogin,
		$elm = $("#successfulLogin");
	me.userName = name;
    (!$elm || 0 >= $elm.length) ? showModal(msg, 3000) : $elm.html(msg);
	me.$backButton.hide();
	me.$nextButton.hide();
};

loginObj.prototype.loginAthlete = function coreLoginObjLoginAthlete() {
	var a = "age=" + this.age,
		b = "weight=" + this.weight,
		c = "gender=" + (1 == this.gender ? "m" : "f"),
		d = "name=" + (null != this.userName ? this.userName : this.userId),
		e = "id=" + this.userId,
		f = "punit=" + this.preferredUnit,
		g = "fbuid" + this.fbuid,
		h = "uuid=" + athleteObj.get_session_id(),
		me = this;

	athleteObj.set_age(this.age);
	athleteObj.set_is_male(this.gender);
	athleteObj.set_weight(this.weight);
	athleteObj.set_punit(this.preferredUnit);
	athleteObj.set_user_id(String(this.userId));
	athleteObj.set_is_aboard(1);
	// Make sure to do this AFTER user_id is set in athleteObj.
	retrieveTvFavorites();
	setTimeout(function() { 
		// If there's a nextView, tell OEM not to hide us upon signing in.
		var argList = ["op=user", "resp=netpulse", a, b, c, d, e];
        if (viewManager.nextView) {
            argList.push("action=nohide");
        }
        if (controlObj.isInternationalEnabled()) {
            argList.push(f);
        }
        if (me.fbuid) {
            argList.push(g);
        }
        if (me.useUuidOnLogin) {
            argList.push(h);
        }
		nsSend(argList.join(","));
	}, this.loginRespDelay);
	this.$pass_container = this.$mainContainer.find('#pass_container').hide();
	fbUserId = this.fbuid;
};

// TODO refactor basic keyboard input
// basic keyboard input
loginObj.prototype.keyboardInput = function coreLoginObjKeyboardInput(e) {
	var me = this;
	if ("userIDKeyboard" == e.target.parentNode.id) {
		var b = $("#id_login_userid"),
			val = b.attr("value"),
			maxLength = parseInt(b.attr("maxlength"));
        maxLength = ((maxLength > 0) ? ((maxLength < 15) ? maxLength : 10) : 10);
		var a = $.trim(e.target.innerHTML);
		if('✔' == a || 'enter' == a || $(e.target).hasClass("enter"))
			me.validateUserId(b.val());
		else
			b.attr("value", (('←' == a) ? val.substring(0, val.length - 1) : (maxLength > val.length) ? val + a : val));
	}
};

// basic keyboard input
loginObj.prototype.userPasscodeKeyboardInput = function coreLoginObjUserPasscodeKeyboardInput(e){
	var me = this;
	if ("userPasscodeKeyboard" == e.target.parentNode.id) {
		var b = $("#userPasscodeInput"),
			val = b.attr("value"),
		    a = $.trim(e.target.innerHTML);

		if ('✔' == a || 'enter' == a || $(e.target).hasClass("enter"))
			me.validateUserPasscode(b.val());
		else
			b.attr("value", (('←' == a) ? val.substring(0, val.length - 1) : (4 > val.length) ? val + a : val));
            if (b.attr("value").length == 4) me.validateUserPasscode(b.val());
	}
};

loginObj.prototype.resetUserPasscode = function coreLoginObjResetUserPasscode() {
    var me = this;
    // reset input field and vars
    $("#userPasscodeInput").attr("value", "");
    $("#setNewUserPasscodeInput").attr("value", "");
    me.userPasscodeCheck = null; 
    me.userPasscode = null;
    //reset input label
    $("#userPasscodeLabel").html(IDS_LANGUAGE.register["newPasscodeLabel"]);
};

// account migration keyboard input
loginObj.prototype.setNewUserPasscodeKeyboardInput = function coreLoginObjSetNewUserPasscodeKeyboardInput(e){
    var me = this;

    if ( e.target.parentNode.id == "setNewUserPasscodeKeyboard" ) {
    var b = $("#setNewUserPasscodeInput"),
        a = $.trim(e.target.innerHTML);
        // if user clicks delete button, reset userpasscode
        if('✔' == a || 'enter' == a || a == 'OK' || $(e.target).hasClass("enter")){
            me.setNewUserPasscode(b.attr("value"));
        } else if (a == '←'){
            me.resetUserPasscode();
            showModal(IDS_LANGUAGE.register["createPasscodeReset"] + " " + IDS_LANGUAGE.register["passcodeEmpty"], 3000);
        } else {
            b.attr("value", (b.attr("value").length < 4) ? b.attr("value") + a : b.attr("value"));
            if (b.attr("value").length == 4) {
                me.setNewUserPasscode(b.attr("value"));
            }
        }
    }
};

loginObj.prototype.resetValues = function coreLoginObjResetValues() {
	user.id = null;
	user.email = null;
	user.password = null;
	user.gender = null;
	user.dob = null;
	user.workoutHistory.total = null;
	user.workoutHistory.workouts = [];

	this.currentPage = 4;
	this.loginAttempts = 0;
	this.passwdImageCount = 48; // Should be 48 TODO
	this.rndImgList = null;
	this.userPasscode = null;
	this.selectedPasswordImage = -1;
	this.userPassword = -1;
	this.userName = null;
	this.userId = null;
	this.age = 0;
	this.weight = 0;
	this.gender = 1;
	this.preferredUnit = "I";
	
	//empty all input fields
	$('#id_login_userid').val('');
	$('#userPasscodeInput').val('');
	$('#setNewUserPasscodeInput').val('');
	// remove selected image password and background border for selected image
	$('#selectedImage').html('');
	$('li.passwordBackground').removeClass('selected');
};

//
// Check the entered user ID to make sure it's valid.
//
// Error return values - badlyFormatted, notRegistered, requestError
//
loginObj.prototype.validateUserId = function coreLoginObjValidateUserId(userId) {
	var me = this;

	// Make sure it's a number
	userId = new String(userId);
	this.userId = userId;

	if (!configObj.isInstalled()) {
		showModal(IDS_LANGUAGE.login["UninstaledMachine"], 3000);
		return;
	}

	if (!(this.userId) || this.userId == undefined || this.userId == "" || isNaN(userId)) {
		showModal(IDS_LANGUAGE.login["EnterUserIdUpTo14"], 3000);
		this.resetValues();
		return;
	}

	if (machineObj.getMachineDemoNetwork()) {
		this.userPasscode = 3366;  // Set a bogus password so that we end up calling validateUserPasscode instead of setNewUserPasscode
		this.slidingBox.scrollNext();
		this.showHideNextBackButtons();
		return;
	}

	var src = machineObj.getServerLocation() + "n5ipicture.jsp?id=" + this.userId + "&mid=" + configObj.getMid() + ((this.oem != null) ? ("&oem=" + this.oem) : "");

	dirtyHarry(src, function(data) {
			me.userIdCallback(data);
			}, function(data) {
			showModal(IDS_LANGUAGE.login["NetworkCommunicationError"], 3000);
			});

};

// XID Validate user passcode
loginObj.prototype.validateUserPasscode = function coreLoginObjValidateUserPasscode(passcode) {
	var me = this;
	var userPasscode = passcode;

		if (userPasscode == "") {
			showModal(IDS_LANGUAGE.register["passcodeEmpty"], 3000);
			this.loginAttempts++;
            this.resetUserPasscode();
			if(this.loginAttempts >= 3){
				this.resetValues();
				showModal(IDS_LANGUAGE.login["3IncorrectPasscodes"], 3000);
				// jump to login home
                me.back();
			}
		} else if (userPasscode.length < 4) {
			showModal(IDS_LANGUAGE.register["passcodeMustBe4Digits"], 3000);
			this.loginAttempts++;
            this.resetUserPasscode();
			if(this.loginAttempts >= 3){
				this.resetValues();
				showModal(IDS_LANGUAGE.login["3IncorrectPasscodes"], 3000);
				// jump to login home
                me.back();
			}		
		} else {
				this.userPasscode = userPasscode;

				if (!(this.userId) || this.userId == undefined || this.userId == "") {
					var errorUrl = machineObj.getServerLocation() + "n5ierror.jsp?mid="
						+ configObj.getMid() + "&priority=error"
						+ "&summary=Login User ID Undefined"
						+ "&message=User ID set to undefined... Nooooooo...";
					dirtyHarry(errorUrl);
					showModal(IDS_LANGUAGE.login["PleaseTryAgainError"], 3000);
					me.back();
					return;
				}

				// If set to demo mode, grab n5ilogin.xml, otherwise assemble a proper request
				var src;
				if(machineObj.getMachineDemoNetwork())
					src = DIR_ROOT + "/www/demo/n5ilogin.xml";
				else{
					switch(configObj.getMid()){
					case "UNINSTALLED":
						var newHostName = "NETPULSE";
						src = machineObj.getSecuredServerLocation() + "n5ilogin.jsp?machine_id=" + newHostName + "&id=" + this.userId + "&pic=" + this.userPassword + "&uuid=" + athleteObj.get_session_id() + "&passcode=" + this.userPasscode;
						break;
					default:
						src = machineObj.getSecuredServerLocation() + "n5ilogin.jsp?machine_id=" + configObj.getMid() + "&id=" + this.userId + "&pic=" + this.userPassword + "&uuid=" + athleteObj.get_session_id() + "&passcode=" + this.userPasscode;
						break;
					}
				}	
				dirtyHarry(src, function(data) {
					me.userPasscodeCallback(data);
				}, function() {
					showModal(IDS_LANGUAGE.login["NetworkCommunicationError"], 3000);
				});
		}
};

// Validate the entered password with the previously validated user id.
// Error return values - badlyFormatted, invalid, requestError, responseError
loginObj.prototype.validatePassword = function coreLoginObjValidatePassword() {
	var me = this;

	// Make sure it's a number
	var password = new Number(this.selectedPasswordImage);

	// Don't allow empty userId
	if (!(this.userId) || this.userId == undefined || this.userId == "") {
		var errorUrl = machineObj.getServerLocation() + "n5ierror.jsp?mid="
			+ configObj.getMid() + "&priority=error"
			+ "&summary=Login User ID Undefined"
			+ "&message=User ID set to undefined... Nooooooo...";
		dirtyHarry(errorUrl);
		showModal(IDS_LANGUAGE.login["PleaseTryAgainError"], 3000);
		this.slidingBox.scrollJump(0);
		this.showHideNextBackButtons();
	}

	// Continue forward if password matches or we are in demo mode
	if (this.selectedPasswordImage == this.userPassword || machineObj.getMachineDemoNetwork()){
		// continue to passcode migration
		this.slidingBox.scrollNext();
		this.showHideNextBackButtons();
	} else {
        // Don't allow empty passwords to be sent to the server
		(isNaN(password) || password < 0) ? showModal(IDS_LANGUAGE.login["NoPasswordSelected"], 3000) : showModal(IDS_LANGUAGE.login["IncorrectPassword"], 3000);

		this.loadRndPasswdImgs();
		this.drawRndPasswdImgs();
		this.loginAttempts++;
		this.selectedPasswordImage = -1;
		$('#loginView .selectionBox').html('');
		$(this.previousSelection).removeClass('selected');

		if (this.loginAttempts >= 3) {
			showModal(IDS_LANGUAGE.login["3IncorrectPasswordsEntered"], 3000);
			// jump to login home
            me.back();
		}
	}
};

loginObj.prototype.setNewUserPasscode = function coreLoginObjSetNewUserPasscode(passcode) {
	var me = this;

    if (passcode == "") { // this should never show up since you can't click enter with the passcode input field empty
        (me.userPasscodeCheck != null) ? showModal(IDS_LANGUAGE.register["createPasscodeConfirm"] + " " + IDS_LANGUAGE.register["passcodeMustBe4Digits"], 2000) :  showModal(IDS_LANGUAGE.register["passcodeEmpty"], 3000);
        return;
    } else if (passcode.length < 4) {
        (me.userPasscodeCheck != null) ? showModal(IDS_LANGUAGE.register["createPasscodeConfirm"] + " " + IDS_LANGUAGE.register["passcodeMustBe4Digits"], 2000) : showModal(IDS_LANGUAGE.register["passcodeEmpty"], 3000);
    } else { // if (passcode.length == 4) 

        if (me.userPasscodeCheck == null) {  // if first input
            // set newUserPasscode to 4 digits entered
            me.userPasscodeCheck = passcode;
            // modal requesting confirmation passcode
            showModal(IDS_LANGUAGE.register["createPasscodeConfirm"], 2000);
            // pause before clearing input field 
            setTimeout(function() {
                $("#setNewUserPasscodeInput").attr("value", "");
                // change label to "Confirm Passcode"
                $("#userPasscodeLabel").html(IDS_LANGUAGE.register["confirmPasscodeLabel"]);
            }, 2000);
            return;

        } else if (me.userPasscodeCheck != null) { //if 2nd input
            if (me.userPasscodeCheck != passcode){
                showModal(IDS_LANGUAGE.register["createPasscodeFail"], 3000);
                setTimeout(function() {
                    me.resetUserPasscode();
                }, 2500);
                return;
            } else {
                me.userPasscode = passcode;
                var src;	
                src = machineObj.getServerLocation() + "updatePasscode.jsp?" + "&id=" + this.userId + "&pic=" + this.userPassword + "&newPasscode=" + this.userPasscode;		
                // ajax call for update passcode
				dirtyHarry(src, function(data) {
					me.validateUserPasscode(me.userPasscode);
				}, function() {
                        showModal(IDS_LANGUAGE.login["NetworkCommunicationError"], 3000);
				});
            }
        } 
    }
};

loginObj.prototype.userIdCallback = function coreLoginObjUserIdCallback(responseXML) {
	var _this = this, response = responseXML.getElementsByTagName('response')[0];

		if (response.getAttribute("pic") != "NOT FOUND") {
			if (response.getAttribute("passcode") == "true") {
				// go to passcode login	
					this.slidingBox.scrollTo(1);
			} else if (response.getAttribute("passcode") == "false" && response.getAttribute("pic") != "-1"){

				// go to image password verification then passcode migration
				this.slidingBox.scrollJump(2);
				
				// set correct user password to prepare for validation
				this.userPassword = response.getAttribute("pic");

				// load and draw password images
				this.loadRndPasswdImgs();
				this.drawRndPasswdImgs();

				// Accommodate LifeFitness Terms of Service flag
				this.showTOS = response.getAttribute("show_oem_tos");

			} else if (response.getAttribute("passcode") == "false" && response.getAttribute("pic") == "-1"){
				// go to passcode migration directly
				this.slidingBox.scrollJump(3);
			}
		} else {
		showModal(IDS_LANGUAGE.login["ThereWasLoginProblem"], 3000);
		this.resetValues();
        }
		this.showHideNextBackButtons();
};

loginObj.prototype.userPasscodeCallback = function coreLoginObjUserPasscodeCallback(responseXML) {
	if (responseXML.getElementsByTagName('response')[0].getAttribute("code") == "0") {
		this.userInfoCallback(responseXML);
	} else {
		showModal(IDS_LANGUAGE.login["ThereWasLoginProblem"], 3000);
        writeLog('info', "code=" + responseXML.getElementsByTagName('response')[0].getAttribute("code") + " : desc=" + responseXML.getElementsByTagName('response')[0].getAttribute("desc"));
		this.loginAttempts++;
        // clear input field
        this.resetUserPasscode();

		if(this.loginAttempts >= 3){
			this.resetValues();
			// jump to login home
			this.slidingBox.scrollJump(0);
			this.showHideNextBackButtons();
			showModal(IDS_LANGUAGE.login["3IncorrectPasscodes"], 3000);
		}
	}
};

loginObj.prototype.userInfoCallback = function coreLoginObjUserInfoCallback(responseXML) {
	var _this = this;
	if (responseXML.getElementsByTagName('response')[0].getAttribute("code") == "0") {
		writeLog("info", "Login successful! NPUUID: " + athleteObj.get_session_id());
		
		// reset playlist
		userCustomPlaylist.reset();

		var r = responseXML.getElementsByTagName('personal')[0];

		// Save the users data to shared memory
		athleteObj.set_user_id(String(this.userId));
		user.id = window.regUser = this.userId;

		athleteObj.set_age(r.getAttribute("age"));
		window.regAge = user.age = this.age = r.getAttribute("age");

		athleteObj.set_weight(r.getAttribute("weight"));
		window.regWeight = user.weight = this.weight = r.getAttribute("weight");

		this.preferredUnit = (r.getAttribute("punit")) ? r.getAttribute("punit") : "I";
		athleteObj.set_punit(this.preferredUnit);

		this.gender = ('M' == r.getAttribute("gender")) ? SZ_DEFINE_MALE : SZ_DEFINE_FEMALE;
		athleteObj.set_is_male(this.gender);
		window.regGender = user.gender = this.gender;

		user.firstName = r.getAttribute("first");
		if (!user.firstName || user.firstName == "null") { user.firstName = null; }
		window.regFirst = user.firstName;

		user.lastName = r.getAttribute("last");
		if (!user.lastName || user.lastName == "null") { user.lastName = null; }
		window.regLast = user.lastName;

		user.secondaryDaysRemaining = secondaryID_daysRemaining = null;

		// if the user needs to connect a secondary id number, set flag to prompt for it.
		$(responseXML).find("membershipId").each(function() {
			user.secondaryDaysRemaining = secondaryID_daysRemaining = $(this).attr("days");
		});

		// set up user's music video playlist
		if (responseXML.getElementsByTagName('playlist').length > 0)
			this.setUpMusicVideoPlaylist(responseXML.getElementsByTagName('playlist')[0]);

		// set up targeted ads TODO: SET UP LATER!!!!
		if (responseXML.getElementsByTagName('adunits').length > 0)
			adMan.setUpTargetedAds(responseXML.getElementsByTagName('adunits')[0]);

		if (responseXML.getElementsByTagName('email').length > 0)
			athleteObj.set_email(responseXML.getElementsByTagName('email')[0].textContent);

		window.userIsLoggedIn = user.userIsLoggedIn = true;
		this.loginAthlete();

        try {
            // this will not work in IE (Who cares about IE!)
            var xmlString = (new XMLSerializer()).serializeToString(responseXML);
            nsSendXML(xmlString);
        }
        catch (e) {
            popError(e);
        }
        
        if (this.$mainContainer && this.$mainContainer.length) {
            this.$mainContainer.addClass('signedIn');
        }

        var storage = responseXML.getElementsByTagName('datastore');

        if(storage.length == 0){
        	fbUserId = null;
        	fbShowPic = false;
        }else{
	        for(var i=0; i< storage.length; i++){
	        	var typeStorage = storage[i].getAttribute('type');
	        	if(typeStorage == 'oem'){
	        		
	        	}else{
	        		var whichFB = storage[i].getElementsByTagName('data');
	        		for(var j=0; j<whichFB.length; j++){
	        			var whichFbName = whichFB.item(j).getAttribute('name');
						if(whichFbName == 'fbuid'){
							var whichFbValue = whichFB.item(j).getAttribute('value');
							fbUserId = whichFbValue;
						}else if(whichFbName == 'showfbpic'){
							var whichPicValue = whichFB.item(j).getAttribute('value');
							fbShowPic = whichPicValue;
						}
	        		}
	        	}
	        }
    	}

        
        fbConnect.isFbUser();
		this.displayWelcomeMsg();
		this.slidingBox.scrollNext();
		this.showHideNextBackButtons();
		this.resetValues();
		if (responseXML.getElementsByTagName('showProfilePic').length > 0) {
			user.hideProfilePic = responseXML.getElementsByTagName('showProfilePic')[0].textContent.trim() == "0" ? true : false;
		}
		this.setTimeout(function() { adMan.ads["startup_video"].showStartupVideoView(viewManager.nextView); }, 2000);
	} else
		showModal(IDS_LANGUAGE.login["ThereWasLoginProblem"] + responseXML.getElementsByTagName('response')[0].getAttribute("desc"), 3000);
};

// set up custom playlist
loginObj.prototype.setUpMusicVideoPlaylist = function coreLoginObjSetUpMusicVideoPlaylist(xml) {
	userCustomPlaylist.setTitle($(xml).attr("name"));
	userCustomPlaylist.shuffle = eval($(xml).attr("shuffle"));
	var videos = $(xml).attr("mediaIds").split(",");

	if (!mediaXML) { // no content: can't do a playlist
		return;
	}

	for ( var i = 0, ilen = videos.length; i < ilen; i++ ) {
		for ( var j = 0, jlen = mediaXML.playlist.length; j < jlen; j++ ) {
			var playlistItem = mediaXML.playlist[j];
			if (parseInt(videos[i]) == parseInt(playlistItem.id)) {
				userCustomPlaylist.add(playlistItem);
				break;
			}
		}
	}
};

// randomize passwords
loginObj.prototype.loadRndPasswdImgs = function coreLoginObjLoadRndPasswdImgs() {
	var arrayImgIds = [];

	for (var i = 0; i < this.passwdImageCount; i++) {
		arrayImgIds[i] = i;
	}

	var j = 0, foundUserPassword = false, imageList = [];

	while (arrayImgIds.length > 0) {
		if (j == this.imgMatrixSize)
			break;
		var number = Math.floor(Math.random() * arrayImgIds.length);
		if (parseInt(arrayImgIds[number]) == parseInt(this.userPassword))
			foundUserPassword = true;
		imageList[j] = arrayImgIds[number];
		arrayImgIds.splice(number, 1);
		j++;
	}

	if (!foundUserPassword && !machineObj.getMachineDemoNetwork()) {
		var number = Math.floor(Math.random() * imageList.length);
		imageList[number] = this.userPassword;
	}

	this.rndImgList = imageList;
};

// randomize passwords
loginObj.prototype.loadRndPasswdImgsNoPassword = function coreLoginObjLoadRndPasswdImgsNoPassword() {
	var arrayImgIds = [];

	for (var i = 0; i < this.passwdImageCount; i++) {
		arrayImgIds[i] = i;
	}

	var j = 0, imageList = [];

	while (arrayImgIds.length > 0) {
		if (j == this.imgMatrixSize)
			break;
		var number = Math.floor(Math.random() * arrayImgIds.length);
		imageList[j] = arrayImgIds[number];
		arrayImgIds.splice(number, 1);
		j++;
	}

	this.rndImgList = imageList;
};

loginObj.prototype.drawRndPasswdImgs = function coreLoginObjDrawRndPasswdImgs() {
	var _this = this;

	$('#randomImages').find('li').each(function(i) {
		$(this).find('div').attr('id', 'icon_' + _this.rndImgList[i]);
	});

	// Skin-specific binding and update
	this.selectedPasswordImage = (-1); // reset password for each attempt.
	$('#randomImages').bind('click', function(e) {
		_this.selectPasswordImg(e);
	});
	$('#selectedImage').html('<div></div>');
};

loginObj.prototype.selectPasswordImg = function coreLoginObjSelectPasswordImg(e) {
    if (this.previousSelection != null) {
        $(this.previousSelection).removeClass('selected');
    }
    this.currentSelection = e.target.parentNode;
    $(this.currentSelection).addClass('selected');
    this.currentlySelectedId = e.target.id.split('_');
    $('#selectedImage > div').removeClass().addClass(e.target.id);
    $("#selectedPass > div").removeClass().addClass(e.target.id);
    // this is where the icon id is stored. 
    this.selectedPasswordImage = this.currentlySelectedId[1];
    this.previousSelection = e.target.parentNode;
    try {
        // update selected password
        var selectionBoxHtml = (this.selectedPasswordImage) ? IDS_LANGUAGE.login.selection
            + '<ul id="randomImages"><li class="passwordBackground"><div id="icon_'
            + this.selectedPasswordImage + '"></div></li></ul>'
            : '';
        $("#loginView .selectionBox").html(selectionBoxHtml);
    } catch(err) {
        popError(err);
    }
};

loginObj.prototype.togglePasswordImages = function coreLoginObjTogglePasswordImages(e) {
	var me = this;

	if (e.target.id != null && e.target.id != "imageCategoryContainer") {
		$('#imageCategoryContainer div').removeClass('selected');
		var target = e.target.id.replace("String", "Button"), trgId = ('#imageCategoryContainer #' + target), $trg = $(trgId);
		me.$animalImages.hide();
		me.$fruitsImages.hide();
		me.$sportsImages.hide();
		me.$otherImages.hide();
		switch (target) {
			case "animalButton":
				me.$animalImages.show();
				break;
			case "fruitButton":
				me.$fruitsImages.show();
				break;
			case "sportsButton":
				me.$sportsImages.show(); 
				break;
			case "otherButton":
				me.$otherImages.show();
				break;
		}
		$trg.addClass('selected');
	}
};

loginObj.prototype.showXIDlogo = function coreLoginObjShowXIDlogo(idslanguage) {
    return sprintf(idslanguage, '<span class="xidSpan"><img class="xidImg" src="images/xid_logo.png" /></span>');
};

loginObj.prototype.htmlInternationalString = function coreLoginObjHtmlInternationalString() {
	var me = this;
	$('#backString').html(IDS_LANGUAGE.login["Back"]);
	$('#nextString').html(IDS_LANGUAGE.login["Next"]);
	$('#passwordDialogString').html(IDS_LANGUAGE.login["passwordDialog"]);
	$('#selectimageString').html(IDS_LANGUAGE.login["SelectImage"]);
	$('#userIdString h1').html(IDS_LANGUAGE.login["userId"]);
	$('#registerString').html(IDS_LANGUAGE.login["register"]);
	$('#cancelString').html(IDS_LANGUAGE.login["Cancel"]);
	$('#isRegisteredString').html(IDS_LANGUAGE.login["IsRegistered"]);
	$('#yetUnregisteredString').html(IDS_LANGUAGE.login["Yet Unregistered"]);

	$('#create').html(this.showXIDlogo(IDS_LANGUAGE.login["createAnAccount"]));

// XID - new additions
	$("#passcodeString").html(IDS_LANGUAGE.register["passcodeString"]);
	$("#passcodeLogin h1").html(IDS_LANGUAGE.login["passcodeLogin"]);
	$("#passcodeMigrationTitle h1").html(IDS_LANGUAGE.login["passcodeMigrationTitle"]);
	$("#passcodeMigrationKeypadTitle").html(IDS_LANGUAGE.register["regSteps3to6Passcode"]);
	$("#passcodeMigration").html(IDS_LANGUAGE.login["passcodeMigration"]);
    $("#userPasscodeLabel").html(IDS_LANGUAGE.register["newPasscodeLabel"]); // "New Passcode"
};



