/*=====================================================================
    TV View Object (CORE) | views/tv/js.js
=====================================================================*/

function tvObj(name, type, vm) {
	this.name = name;
	this.type = type;
	this.vm = vm;
	this.originView = "";
	
	this.favorites = {
		data : { items : Array() },
		itemsPerPage : 7,
		slideBox : null
	};
	
	this.enabledChannels = {
		data : { items : Array() },
		itemsPerPage: 14,
		slideBox : null
	};

	// Maximum size of text to fit into the TV buttons.  Currently defined to be the same as the Channel Icons (110px x 110px) 
	// I changed this to be 140x140 so that we can have about a 5px margin in the worst case - Marc 
	this.buttonContentSize = {
		height: 140,
		width: 140
	};
	
//	this.data = {
//		favorites : { items : Array() }, // data array for slidingBox (Favorites)
//		channels : { items : Array() }  // data array for slidingBox (available Channels)
//	}
//	this.itemsPerPage = {
//		favorites : 8,
//		channels : 24
//	}
//	this.slideBoxFavorites; // slidingBox for favorites
//	this.slideBoxChannels;  // slidingBox for channels
 }

tvObj.prototype = new app;

tvObj.prototype.init = function() {
	try {
		// Get channel list
		var channelList = tvConfig.getChannelList(true);
		if (channelList.length < 1) 
			throw new Error("No TV channels enabled");
		
	} catch(e) {
		writeLog('error', 'No TV channels found - disabling TV.');
		this.disabled = true;
	}
};

tvObj.prototype.preShowHook = function coreTvObjPreShowHook() {};

tvObj.prototype.show = function() {
    this.preShowHook();
    
    // Check previous view name before assigning originView. We don't want to return
    // to login/registration, unless there is no where else to go.
    var viewName = this.vm.getPreviousViewName();
    switch (viewName.toLowerCase()) {
        case "login":
        case "registration":
            break;
        default:
            if (this.vm.isViewAvailable(viewName)) {
                this.originView = viewName;
            }
            break;
    }

	this.bindHTMLElements();
	this.bindSkinSpecificElements();
	
	// Hide this since it's not being used. Since the cue container is created after the content for the
	// vid browser has been filled, the vid browser gets pushed down after the page is rendered.
	this.$favoritesCueContainer.hide();
	
	this.buildLevelsArray();
	this.buildSlideBox();
	
	this.attachEvents();
	this.htmlInternationalString();
    this.postShowHook();
};

tvObj.prototype.postShowHook = function coreTvObjPostShowHook() {};

tvObj.prototype.hide = function hide() {

};

tvObj.prototype.bindHTMLElements = function() {
	this.$container = $("#tvView");
	
	this.$favoritesContainer = this.$container.find("#favorites");
	this.$favoritesSlidingViewPort = this.$favoritesContainer.find("#slidingViewPort");
	this.$favoritesCueContainer = this.$favoritesContainer.find("#cueContainer");
	this.$favoritesVidBrowser = this.$favoritesContainer.find("#vidBrowser");
	this.$favoritesBackButton = this.$favoritesContainer.find("#backButton"); 
	this.$favoritesNextButton = this.$favoritesContainer.find("#nextButton");

	this.$channelsContainer = this.$container.find("#channels");
	this.$channelsHeader = this.$container.find("#channels h2")
	this.$channelsSlidingViewPort = this.$channelsContainer.find("#slidingViewPort");
	this.$channelsCueContainer = this.$favoritesContainer.find("#cueContainer");
	this.$channelsVidBrowser = this.$channelsContainer.find("#vidBrowser");
	this.$channelsBackButton = this.$channelsContainer.find("#backButton");
	this.$channelsNextButton = this.$channelsContainer.find("#nextButton");
};

tvObj.prototype.bindSkinSpecificElements = function () {};

tvObj.prototype.attachEvents = function() {
	// TODO
};

tvObj.prototype.buildLevelsArray = function() {
	var me = this;
	
	// Always do this! Otherwise, duplicate entries may be added to the
	// favorites list. Leaving the tv view will not clear this array.
	this.favorites.data.items = new Array();
	
	var favorites = athleteObj.get_tv_favorites();
	var allChannels = tvConfig.getChannelList(false);
	for (var i=0; i<favorites.length; i++) {
		// Translate channels on favorites to full channel information
		for (var j=0; j<allChannels.length; j++) {
			if (favorites[i] == allChannels[j].channel) {
				allChannels[j].favorite = true;
				me.favorites.data.items.push(allChannels[j]);
				break;
			}
		}
	}
	
	me.favorites.data.items.sort(tvConfig.sortByChannelPosition);

	me.favorites.data.levels = new Array();
	me.favorites.data.levels[0] = {
		drawItem : function(data) {
			// Main button/icon for channel
			var html = '<div class="menu_button'
				+ (data.enabled ? '' : ' disabled')
				+ (data.icon ? ' icon' : data.name ? ' name' : ' channel')
				+ '">';
			if (data.icon) {
				html += '<div style="background-image:url(../images/channelIcons/' + data.icon + ');"></div>';
			}
			html += '<h2>' + (data.name ? data.name : data.channel) + '</h2></div>';
			
			// Button/icon to remove channel from favorites
			html += '<div class="favorite remove"><span>remove</span></div>';

			return html;
		},
		attachEvents : function(slideBox, $box) {
			me.attachChannelEvents(slideBox);
		},
		itemsPerPage : me.favorites.itemsPerPage
	};
	
	me.enabledChannels.data.items = tvConfig.getChannelList(true);
	me.enabledChannels.data.levels = new Array();
	me.enabledChannels.data.levels[0] = {
		drawItem : function(data) {
			// Channel button
			var html = '<div class="menu_button'
				+ (data.icon ? ' icon' : data.name ? ' name' : ' channel')
				+ '">';
			if (data.icon) {
				html += '<div style="background-image:url(../images/channelIcons/' + data.icon + ');"></div>';
			}
			html += '<h2>' + (data.name ? data.name : data.channel) + '</h2></div>';

			// add to/remove from favorites sub-button
			var label =  ( ((typeof(data.favorite) != "undefined") && data.favorite) ? 'remove' : 'add' );
			html += '<div class="favorite '
				// Indicate channel is already in favorites, 
				+ label + (athleteObj.get_user_id() == "" ? ' disabled' : '') + '"><span>' + label + '</span></div>';
			return html;
		},
		attachEvents : function(slideBox, $box) {
			me.attachChannelEvents(slideBox);
		},
		itemsPerPage : me.enabledChannels.itemsPerPage
	};
};

tvObj.prototype.buildSlideBox = function() {
	var me = this;
	me.favorites.slideBox = new slidingBox(
		me.favorites.data,
		me.$favoritesSlidingViewPort,
		me.$favoritesVidBrowser,
		me.$favoritesBackButton,
		me.$favoritesNextButton,
		1,
		200,
		null,
		me.$favoritesCueContainer,
		true
	);
	if (me.$favoritesVidBrowser.html() == "") {
		if (athleteObj.get_user_id() > "") {
			this.$favoritesVidBrowser.html('<li><p></p></li>');
		} else {
			this.$favoritesVidBrowser.html('<button id="login" class="button"><span></span></button>');
			this.$favoritesVidBrowser.find("button#login").bind("click." + me.name, function() {
				me.vm.nextView = "tv";
				me.vm.show("login");
			});
		}
	}
	// Adjust the Channel favorites to fit channel name
	me.favorites.slideBox.$box.find(".menu_button").not(".icon").each(me.fitTextToButton);			

	// Adjust all future calls to redraw the Channel favorites as well
	me.favorites.slideBox.baseDrawList = me.favorites.slideBox.drawList;
	me.favorites.slideBox.drawList = function() {
		me.favorites.slideBox.baseDrawList();
		me.favorites.slideBox.$box.find(".menu_button").not(".icon").each(me.fitTextToButton);	
	};

	me.enabledChannels.slideBox = new slidingBox(
		me.enabledChannels.data,
		me.$channelsSlidingViewPort,
		me.$channelsVidBrowser,
		me.$channelsBackButton,
		me.$channelsNextButton,
		1,
		200,
		null,
		me.$channelsCueContainer,
		true
	);
	// Adjust the Channel Line-up to fit the channel name
	me.enabledChannels.slideBox.$box.find(".menu_button").not(".icon").each(me.fitTextToButton);
};

tvObj.prototype.fitTextToButton = function(index, button) {
	var view = viewManager.activeObject,
		width = view.buttonContentSize.width,
		height = view.buttonContentSize.height,
		line = button.firstElementChild;
	
	var fontSize = parseInt($(line).css("font-size"));

	// Resize the font so that the entire text fits within its parent container
	var newSize;
	var fontSizeRatio = width / line.scrollWidth;
	if (fontSizeRatio < 1) {
		newSize = (fontSize*fontSizeRatio).toFixed(2);
		line.setAttribute("style", "font-size: "+newSize+"px;");
	}
	
	var fontSizeRatio = height / line.scrollHeight;
	if (fontSizeRatio < 1) {
		newSize = ((newSize || fontSize)*fontSizeRatio).toFixed(2);
		line.setAttribute("style", "font-size: "+newSize+"px;");
	}
};

tvObj.prototype.attachChannelEvents = function(slideBox) {
	var me = this;
	
	slideBox.$box.find("li .menu_button").bind("click." + me.name, function() {
		if (!$(this).hasClass("disabled"))
			me.watch(slideBox);
	});
	slideBox.$box.find("li .favorite").bind("click." + me.name, function() {
		if (!$(this).hasClass("disabled"))
			me.toggleFavorite($(this), slideBox);
	});
};

tvObj.prototype.watch = function(slideBox) {
	var selectedChannel = this.enabledChannels.slideBox.chosenItems[0];
	if (slideBox == this.favorites.slideBox) {
		// Channel was selected from favorites list; get the index of the 
		// selected channel from the enabled channels list.
		for (var i=0; i<this.enabledChannels.slideBox.data.items.length; i++) {
			if (this.favorites.slideBox.data.items[this.favorites.slideBox.chosenItems[0]].channel == this.enabledChannels.slideBox.data.items[i].channel) {
				selectedChannel = i;
				break;
			}
		}
	}
	watchObj.target = {
		"parentView" : this.vm.currentView,
		"channel" : this.enabledChannels.slideBox.data,	// Channel List
		"episode" : selectedChannel, // Selected channel
		"showTitle" : false,
	};
	this.vm.show("watch");
};

tvObj.prototype.toggleFavorite = function(element, slideBox) {
	var channel = slideBox.data.items[slideBox.chosenItems[0]];
	var currentPage = this.favorites.slideBox.currentPage;
	
	if ((typeof(channel.favorite) != "undefined") && channel.favorite) {
		// Currently in favorites; remove
		channel.favorite = false;
		var button = element;
		if (slideBox == this.favorites.slideBox) {
			// Removed via favorites slideBox, so update the channels sub-button, not the favorites sub-button
			var index = -1;
			for (var i=0; i<this.enabledChannels.slideBox.data.items.length; i++) {
				if (channel.channel == this.enabledChannels.data.items[i].channel) {
					index = i;
					break;
				}
			}
			if (index > -1) {
				button = $("#channels #slidingViewPort #vidBrowser").find("input[type=hidden][value=" + index + "]").parent().find(".favorite");
			}
		}
		// change sub-button to indicate not-in-favorites
		$(button).addClass("add")
			.removeClass("remove")
			//.html("<span>add</span>")
			;
		
		// Update data for favorites sliding box
		for (var i=0; i<this.favorites.data.items.length; i++) {
			if (channel.channel == this.favorites.data.items[i].channel) {
				this.favorites.data.items.splice(i, 1);
				currentPage = Math.floor(i / this.favorites.itemsPerPage);
				break;
			}
		}

		// Update global favorites
		athleteObj.remove_tv_favorite( channel.channel );
	} else {
		// Not yet in favorites; add
		channel.favorite = true;
		$(element).addClass("remove")
			.removeClass("add")
			//.html("<span>remove</span>")
			;
		
		// Update data for favorites sliding box
		this.favorites.data.items.push(channel);
		this.favorites.data.items.sort(tvConfig.sortByChannelPosition);
	
		// Calculate page to show
		for (var i=0; i<this.favorites.data.items.length; i++) {
			if (this.favorites.data.items[i] == channel) {
				currentPage = Math.floor(i / this.favorites.itemsPerPage);
			}
		}
		// Update global favorites
		athleteObj.add_tv_favorite( channel.channel );
	}
	
	// Redraw favorites sliding box and show affected page
	this.favorites.slideBox.drawLevel(0);
	if (currentPage != this.favorites.slideBox.currentPage) {
		if (this.favorites.data.items.length > 0) {
			var prevDoTransitions = doTransitions;
			doTransitions = false;
			this.favorites.slideBox.scrollTo(currentPage);
			doTransitions = prevDoTransitions;
		}
	}
};

tvObj.prototype.htmlInternationalString = function (){
	var me = this;
	me.$favoritesContainer.find("h1").html(IDS_LANGUAGE.tv['myChannels']);
	me.$container.find("#channels>h2").html(IDS_LANGUAGE.tv['channelGuide']);
	me.$favoritesVidBrowser.find("li p").html(IDS_LANGUAGE.tv['emptyFavorites']);
	me.$favoritesVidBrowser.find("#login span").html(IDS_LANGUAGE.tv['login']);//used in root
	me.$favoritesVidBrowser.find("#login div").html(IDS_LANGUAGE.tv['login']);//used in 7xi and true
};

// Use TV back button to return to last view.
// Usage: this.useBackToMenuButton(this.enabledChannels.slideBox);
tvObj.prototype.useBackToMenuButton = function coreTvObjUseBackToMenuButton(slideBox, param) {
    var me = this;
    if (slideBox) {
        // Save original scrollBack function.
        slideBox.coreScrollBack = slideBox.scrollBack;
        // Hijack scroll functions on first page.
        slideBox.scrollBack = function hijackScrollBack(param) {
            var currentPage = slideBox.currentPage || 0;
            if (currentPage == 0) {
                if (viewManager.getPreviousViewName() == me.originView) {
                    me.originView = "";
                    viewManager.showPreviousView();
                } else {
                    var view = me.originView;
                    me.originView = "";
                    viewManager.show(view);
                }
            } else {
                slideBox.coreScrollBack(param);
            }
        };

        // Hijack hide function, overrides slidingBox.hideScrollBackButton.
        slideBox.hideScrollBackButton = function hijackHideScrollBackButton() { /* Don't hide me. */ };

        // Show back button.
        slideBox.showScrollBackButton();
    }
};

/*===================================================================*/
//@ sourceURL=views/tv/js.js
