/*=====================================================================
	Work-out History Script (CORE) | views/workoutHistory/js.js
=====================================================================*/

function workoutHistoryObj(name, type, vm, htmlFragment) {
    this.name = name;
    this.type = type;
    this.vm = vm;
	this.stats;
	this.distUnits;

    this.htmlFragment = htmlFragment;

    this.bShowControls = true;
    this.selectedRow = null;
    this.selectedRowClass = null;
    this.maxRows = 10;

    this.displayStart = 0;
    this.fetchChunk = 100;
    this.currentPage = 1;
    this.workouts = [];
    this.totalWorkouts = 0;

    this.$workoutDataHolder = null;
}

//  Inherit basic view functions init(), show(), hide()
workoutHistoryObj.prototype = new app;

workoutHistoryObj.prototype.show = function workoutHistoryObjShow() {
    try {
        //
        this.preShowHook();
        
        this.$workoutDataHolder = $("#workoutDataHolder"); //main container for drawTable()
        //
        this.bindHTMLElements();
        //
        this.attachEvents();

        if (userIsLoggedIn) {
            //
            this.fetchRecords();
            // get the most recent workout data for drawTable()
            this.resetData();
            //
            this.fetchRecords(1, this.fetchChunk);
        } else {
            this.drawLoginButton();
        }
        //
        this.postShowHook();
    } catch(e) {
        popError(e);
        return false;
    }

    this.htmlInternationalString();

    return true;
}; 

workoutHistoryObj.prototype.preShowHook = function coreWorkoutHistoryObjPreShowHook() {};

workoutHistoryObj.prototype.postShowHook = function coreWorkoutHistoryObjPostShowHook() {};

workoutHistoryObj.prototype.drawLoginButton = function workoutHistoryObjDrawLoginButton() {
    var me = this;
    if (!userIsLoggedIn) {
        me.$loginButton = $("<button id='loginButton'><span>" + IDS_LANGUAGE["login/regString"] + "</span></button>");
        $("#workoutDataHolder").append(me.$loginButton);

        if (!configObj.isInstalled() && !machineObj.getMachineDemoNetwork()) {
            me.$loginButton.bind("mousedown." + this.name, function () { 
                showModal(IDS_LANGUAGE.installer["notInstalledFeature"], 5000);
            });
        } else {
            me.$loginButton.bind("mousedown." + this.name, function () { 
                viewManager.nextView = viewManager.activeObject.name; 
                viewManager.show("login"); 
            });
        }
    }
};

workoutHistoryObj.prototype.bindHTMLElements = function()
{
    this.$title = $("#header h1");
//    this.$upButtonWorkout = $('#upButtonWorkout');
//    this.$downButtonWorkout = $('#downButtonWorkout');
    this.$backButton = $("#backButton");
    this.$nextButton = $("#nextButton");
    this.$loginButtonWorkout = $("#loginButtonWorkout");
    this.$allTime = $("#allTime");
    this.$mainContainer = $('div#workoutHistoryView');

};

workoutHistoryObj.prototype.attachEvents = function()
{
    var me = this;
    return me;

};

workoutHistoryObj.prototype.buildMenuData = function()
{
    var me = this;

    me.menuData = [];
    me.menuData.levels = [];

    me.menuData.levels[0] =
    {
        drawItem :
            function(data)
            {
                return("<div class='"+data.thumbnail+"' ></div><h2>"+data.title+"</h2>");
            },
        attachEvents: 
            function(slideBox, $box)
            {
                me.$title.html("Workout History");
                me.updateScrollButtonLabels(slideBox);
                slideBox.attachEventsDefault();
            },
        scrollBack: function(e) { me.slidingBox.scrollBack(e); me.updateScrollButtonLabels(me.slidingBox); },
        scrollNext: function(e) { me.slidingBox.scrollNext(e); me.updateScrollButtonLabels(me.slidingBox); },
        itemsPerPage : 4
    };

    me.menuData.levels[1] =
    {
        drawItem :
            function(data)
            {
                return(data.html);
            },
        attachEvents: 
            function(slideBox, $box)
            {
                me.$title.html("Workout History - "+slideBox.data.items[slideBox.chosenItems[0]].title);
                me.updateScrollButtonLabels(slideBox);
                me.updateStats("calories");
            },
        scrollBack: function(e) { me.slidingBox.scrollBack(e); me.updateScrollButtonLabels(me.slidingBox); },
        scrollNext: function(e) { me.slidingBox.scrollNext(e); me.updateScrollButtonLabels(me.slidingBox); },
        itemsPerPage : 1
    };

    var htmlStr = $("#historyPageContents").html();

    this.menuData.items = [];

    this.menuData.items[0] =
    {
        "title" : "This Week",
        "thumbnail" : "weekIcon"
    };
    this.menuData.items[0].items = [];
    this.menuData.items[0].items[0] =
    {
        "title" : "Weekly Stats",
        "html"  :  htmlStr
    };

    this.menuData.items[1] =
    {
        "title" : "This Month",
        "thumbnail" : "monthIcon"
    };
    this.menuData.items[1].items = [];
    this.menuData.items[1].items[0] =
    {
        "title" :  "Monthly Stats",
        "html"  :  htmlStr
    };

    this.menuData.items[2] =
    {
        "title" : "This Year",
        "thumbnail" : "yearIcon"
    };
    this.menuData.items[2].items = [];
    this.menuData.items[2].items[0] =
    {
        "title" :  "Yearly Stats",
        "html"  :  htmlStr
    };

    this.menuData.items[3] =
    {
        "title" : "All Time",
        "thumbnail" : "allTimeIcon"
    };
    this.menuData.items[3].items = [];
    this.menuData.items[3].items[0] =
    {
        "title" :  "All Time Stats",
        "html"  :  htmlStr
    }; 
};

workoutHistoryObj.prototype.fetchRecords = function workoutHistoryObjFetchRecords() {
    var me = this;

    //  If set to demo mode
    if(machineObj.getMachineDemoNetwork())
    {
    	//alert("un-first alert");
        var demoString = DIR_ROOT + '/www/demo/n5ihistory.json';
    	this.$jqxhr = $.getJSON(
            demoString,
            function(data) { try { me.getHistoryCallback(data); } catch(e) { popError(e); } }
        );
        return;
    }

    $.ajaxSetup({
    	  "error":function(XMLHttpRequest, textStatus, errorThrown) {
    	  	//alert (' an error occured - please use Firebug console to view the details ')
    	  	if (window.console && window.console.log){
    	  		 console.log(textStatus);
    	  		 console.log(errorThrown);
    	  		 console.log(XMLHttpRequest.responseText);
    	  		 console.log(XMLHttpRequest);
    	  		 popError(errorThrown);
    	  	}
    	}
    });


	var userId = athleteObj.get_user_id();
	var reqURL = machineObj.getServerLocation() + "n5ihistory.jsp?" + "user_id="+ userId +"\&machine_id="+configObj.getMid()+"\&beg=1\&end=1000"; 
	dirtyHarry(reqURL, function(data){ me.getHistoryCallback(data);}, null, null, null, null, 'json');
	
	
};

workoutHistoryObj.prototype.getHistoryCallback = function(data) {
	//  If no workout history, bail
	if(!data || data.length == 0)
		return;

	this.stats = data;

	var htmlStr = "<table><thead><tr>" +
			"<th>Date</th>" +
			"<th>Distance</th>" +
			"<th>Duration</th>" +
			"<th>Activity</th>" +
			"<th>Calories</th>" +
			"</tr></thead><tbody>";

	if (this.stats.workouts.length == 0) {
		this.$allTime.html('<center><h2>You currently have no workout history.</h2></center>');
	} else {
	    for(var i=0,w=null;i<this.stats.workouts.length;i++) {
	        w = this.stats.workouts[i];
			this.distUnits = this.stats.workouts[i].distance_units;
		 	if (!this.distUnits || this.distUnits.length < 1) this.distUnits = "miles";

		 	htmlStr += "<tr>";

	        htmlStr += "<td>";
	            w.date = w.date_time.split(" ");
	            htmlStr += (w.date_time== undefined) ? "&nbsp;" : w.date[0];
	        htmlStr += "</td>";

	        htmlStr += "<td>";
	            htmlStr += (w.distance == undefined) ? "0 "+this.distUnits : w.distance+" "+this.distUnits;
	        htmlStr += "</td>";

	        htmlStr += "<td>";
	            htmlStr += (w.duration == undefined) ? "&nbsp;" : this.convertSecsToReadableTime(w.duration);
	        htmlStr += "</td>";

			htmlStr += "<td>";
	            htmlStr += (w.type == undefined) ? "&nbsp;" : w.type;
	        htmlStr += "</td>";

	        htmlStr += "<td>";
	            htmlStr += (w.calories == undefined) ? "0" : w.calories;
	        htmlStr += "</td>";

	        htmlStr += "</tr>";
	    }
	    htmlStr += "</tbody></table>";

	    this.$allTime.html(htmlStr);
	}
};

workoutHistoryObj.prototype.convertSecsToReadableTime = function workoutHistoryObjConvertSecsToReadableTime(timeInt) {
    var secs = Math.floor(timeInt % 60),
        mins = Math.floor(timeInt / 60),
        hour = Math.floor(mins / 60);
    // Convert null values
    if (isNaN(hour)) {
        hour = 0;
    }
    mins = (mins - (hour * 60));
    if (isNaN(mins)) {
        mins = 0;
    }
    if (isNaN(secs)) {
        secs = 0;
    }
    var secsString = (secs < 10) ? '0' + secs : secs,
        minsString = (mins < 10) ? '0' + mins : mins,
        hourString = (hour < 10) ? '0' + hour : hour;
    return(hourString + ":" + minsString + ":" + secsString);
};

workoutHistoryObj.prototype.convertTime = function coreWorkoutHistoryObjConvertTime(timeInt) {
    return(this.convertSecsToReadableTime(timeInt));
};

//These objects are for the drawTable non scrollbar workout history view

workoutHistoryObj.prototype.resetData = function() {
    this.currentPage = 1;
    this.workouts = [];
    this.totalWorkouts = 0;
};

workoutHistoryObj.prototype.nextPage = function coreWorkoutHistoryObjNextPage() {
    try {
        // If we've passed the end, stay on the last page.
        var maxPages = Math.floor(this.totalWorkouts / this.maxRows);
        if (this.currentPage > maxPages) {
            this.currentPage = maxPages;
            return;
        }

        this.currentPage++;

        var bottomOfComingPage = parseInt(this.currentPage * this.maxRows),
            beg = this.workouts.length,
            end = Math.min(parseInt(this.fetchChunk + beg), this.totalWorkouts);

        if (bottomOfComingPage > beg && bottomOfComingPage < this.totalWorkouts) {
            this.fetchRecords(beg, end);
            return;
        }
        this.drawTable();
    } catch (e) {
        popError(e);
    }
};

workoutHistoryObj.prototype.prevPage = function() {
    try {
        this.currentPage--;

        if (this.currentPage < 1)
            this.currentPage = 1;

        this.drawTable();

    } catch (e) {
        popError(e);
    }
};

workoutHistoryObj.prototype.showNoWorkoutsPrompt = function()
{
    var noWorkoutsPrompt =
            "<div id='noWorkoutsYet'>" +
            "<div id='id_lp_message'><h1>" + IDS_LANGUAGE.workout['noworkouts'] + "</h1></div>" +
            "</div>";

    this.$workoutDataHolder.html(noWorkoutsPrompt);
};

// Use drawTables function to create non scrollbar based table.
workoutHistoryObj.prototype.drawTable = function coreWorkoutHistoryObjDrawTable() {
    try {
        var _this = this,
            beg = (this.currentPage - 1) * this.maxRows,
            end = (beg) + this.maxRows,
            tableString = "",
            i = 0;

        if (end > this.totalWorkouts) {
            end = (0 + this.totalWorkouts);
        }

        this.displayStart = beg;

        tableString += "<div id='wh_wrap'>";
        tableString += "<div id='wh_page'>Displaying " + (beg + 1) + " - " + end + " of " + this.totalWorkouts /*total*/ + "</div>";

        tableString += "<table id='wh'>" +
            "<thead id='wh_head'>" +
                "<tr id='tr_head' class='whRow'>" +
                    "<th id='th_date' class='date'><span>"+
                    IDS_LANGUAGE.workout["date"]+
                    "</span></th>" +
                    "<th id='th_duration' class='dura'><span>";
        tableString+= IDS_LANGUAGE.workout["duration"];

        tableString+= "</span></th>"+
                    "<th id='th_distance' class='dist'><span>"+
                    IDS_LANGUAGE.workout["distance"]+
                    "</span></th>" +
                    "<th id='th_activity' class='type'><span>"+
                    IDS_LANGUAGE.workout["activity"]+
                    "</span></th>" +
                    "<th id='th_calories' class='cals'><span>"+
                    IDS_LANGUAGE.workout["calories"]+
                    "</span></th>" +
                "</tr>" +
            "</thead>";
        tableString += "<tbody id='wh_body'>";

        for (i = beg; i < end; i++ ) {  
            var className = (i % 2 == 0) ? "even" : "odd",
                w = this.workouts[i],
                yr = w.date_time.substr(0, 4),
                mo = w.date_time.substr(5, 2).replace(/0(\d{1,1})/, "$1"),
                dy = w.date_time.substr(8, 2).replace(/0(\d{1,1})/, "$1"),
                datestring = '';

            // need to specifically set each field: Date.UTC() does not
            // calculate day correctly.
            datestring = mo + "/" + dy + "/" + yr;
            className += (i == end - 1) ? " lastRow" : "";

            // Convert null values
            w.distance = (isNaN(w.distance)) ? "" : w.distance;
            w.distance_units = (w.distance_units == null || w.distance_units == "") ? "" : w.distance_units;
            w.calories = (w.calories + "" == "null") ? "0" : w.calories;

            tableString += "<tr id='wh_row_" + w.record.trim() + "' class='whRow " + className + "'>" +
                    "<td class='date'><span>" + datestring + "</span></td>" +
                    "<td class='dura'><span>" + this.convertTime(w.duration) + "</span></td>" +
                    "<td class='dist'><span>" + w.distance + " " + w.distance_units + "</span></td>" +
                    "<td class='type'><span>" + w.type + "</span></td>" +
                    "<td class='cals'><span>" + w.calories + "</span></td>" +
                "</tr>";
        }

        tableString += "</tbody></table>";
        
        if (this.totalWorkouts > this.maxRows) {
            tableString += "<div id='wh_scroll'>" +
                "<div id='wh_prev'><!--/ UP /--></div>" +
                "<div id='wh_next'><!--/ DOWN /--></div>" +
                "</div>";
        }
        
        tableString += "</div>";

        this.$workoutDataHolder.html(tableString);
        
        if (this.totalWorkouts > this.maxRows) {
            this.$workoutDataHolder.find('#wh').removeClass('center');
        } else {
            this.$workoutDataHolder.find('#wh').addClass('center');
        }

        // Need to find prev every time the data table is built.
        this.$workoutPagePrev = this.$workoutDataHolder.find("#wh_prev");
        if (0 != beg) {
            this.$workoutPagePrev.removeClass('disabled').unbind()
                .bind("mousedown." + _this.name, function() { return false; })
                .bind("mouseup." + _this.name, function() { _this.prevPage(); });
        } else {
            this.$workoutPagePrev.addClass('disabled');
        }

        // Need to find next every time the data table is built.
        this.$workoutPageNext = this.$workoutDataHolder.find("#wh_next");
        if (this.totalWorkouts > end) {
            this.$workoutPageNext.removeClass('disabled').unbind()
                .bind("mousedown." + _this.name, function() { return false; })
                .bind("mouseup." + _this.name, function() { _this.nextPage(); });
        } else {
            this.$workoutPageNext.addClass('disabled');
        }
         
    } catch (e) {
        popError(e);
    }
};

workoutHistoryObj.prototype.htmlInternationalString = function coreWorkoutHistoryObjHtmlInternationalString() {

$('#th_date').html(IDS_LANGUAGE.workout["date"]);
$('#th_duration').html(IDS_LANGUAGE.workout["duration"]);
$('#th_distance').html(IDS_LANGUAGE.workout["distance"]);
$('#th_activity').html(IDS_LANGUAGE.workout["activity"]);
$('#th_calories').html(IDS_LANGUAGE.workout["calories"]);
}

